@@ -50,7 +50,6 @@ static void setup_codepages();
 #define EXE_LINKPATH "/proc/curproc/file"
 #endif
 
-extern pthread_key(struct st_my_thread_var*, THR_KEY_mysys);
 
 #define SCALE_SEC       100
 #define SCALE_USEC      10000
@@ -348,8 +347,7 @@ Voluntary context switches %ld, Involuntary context switches %ld\n",
 #endif
  
   /* At very last, delete mysys key, it is used everywhere including DBUG */
-  pthread_key_delete(THR_KEY_mysys);
-  my_init_done= my_thr_key_mysys_exists= 0;
+  my_init_done= 0;
 } /* my_end */
 
 #ifdef DBUG_ASSERT_EXISTS
@@ -466,7 +464,7 @@ PSI_mutex_key key_LOCK_localtime_r;
 
 PSI_mutex_key key_BITMAP_mutex, key_IO_CACHE_append_buffer_lock,
   key_IO_CACHE_SHARE_mutex, key_KEY_CACHE_cache_lock,
-  key_LOCK_alarm, key_LOCK_timer,
+  key_LOCK_timer,
   key_my_thread_var_mutex, key_THR_LOCK_charset, key_THR_LOCK_heap,
   key_THR_LOCK_lock, key_THR_LOCK_malloc,
   key_THR_LOCK_mutex, key_THR_LOCK_myisam, key_THR_LOCK_net,
@@ -485,7 +483,6 @@ static PSI_mutex_info all_mysys_mutexes[]=
   { &key_IO_CACHE_append_buffer_lock, "IO_CACHE::append_buffer_lock", 0},
   { &key_IO_CACHE_SHARE_mutex, "IO_CACHE::SHARE_mutex", 0},
   { &key_KEY_CACHE_cache_lock, "KEY_CACHE::cache_lock", 0},
-  { &key_LOCK_alarm, "LOCK_alarm", PSI_FLAG_GLOBAL},
   { &key_LOCK_timer, "LOCK_timer", PSI_FLAG_GLOBAL},
   { &key_my_thread_var_mutex, "my_thread_var::mutex", 0},
   { &key_THR_LOCK_charset, "THR_LOCK_charset", PSI_FLAG_GLOBAL},
@@ -502,13 +499,12 @@ static PSI_mutex_info all_mysys_mutexes[]=
   { &key_LOCK_uuid_generator, "LOCK_uuid_generator", PSI_FLAG_GLOBAL }
 };
 
-PSI_cond_key key_COND_alarm, key_COND_timer, key_IO_CACHE_SHARE_cond,
+PSI_cond_key key_COND_timer, key_IO_CACHE_SHARE_cond,
   key_IO_CACHE_SHARE_cond_writer, key_my_thread_var_suspend,
   key_THR_COND_threads, key_WT_RESOURCE_cond;
 
 static PSI_cond_info all_mysys_conds[]=
 {
-  { &key_COND_alarm, "COND_alarm", PSI_FLAG_GLOBAL},
   { &key_COND_timer, "COND_timer", PSI_FLAG_GLOBAL},
   { &key_IO_CACHE_SHARE_cond, "IO_CACHE_SHARE::cond", 0},
   { &key_IO_CACHE_SHARE_cond_writer, "IO_CACHE_SHARE::cond_writer", 0},
@@ -524,16 +520,10 @@ static PSI_rwlock_info all_mysys_rwlocks[]=
   { &key_SAFEHASH_mutex, "SAFE_HASH::mutex", 0}
 };
 
-#ifdef USE_ALARM_THREAD
-PSI_thread_key key_thread_alarm;
-#endif
 PSI_thread_key key_thread_timer;
 
 static PSI_thread_info all_mysys_threads[]=
 {
-#ifdef USE_ALARM_THREAD
-  { &key_thread_alarm, "alarm", PSI_FLAG_GLOBAL},
-#endif
   { &key_thread_timer, "statement_timer", PSI_FLAG_GLOBAL}
 };
 
