@@ -19,11 +19,7 @@
 /* subselect Item */
 
 #include "item.h"
-#ifdef USE_PRAGMA_INTERFACE
-#pragma interface			/* gcc class implementation */
-#endif
-
-#include <queues.h>
+#include "sql_queue.h"
 
 class st_select_lex;
 class st_select_lex_unit;
@@ -297,7 +293,8 @@ public:
 /* single value subselect */
 
 class Item_cache;
-class Item_singlerow_subselect :public Item_subselect
+class Item_singlerow_subselect :public Item_subselect,
+                                public Type_extra_attributes
 {
 protected:
   Item_cache *value, **row;
@@ -322,6 +319,14 @@ public:
   bool get_date(THD *thd, MYSQL_TIME *ltime, date_mode_t fuzzydate) override;
   const Type_handler *type_handler() const override;
   bool fix_length_and_dec() override;
+  Type_extra_attributes *type_extra_attributes_addr() override
+  {
+    return this;
+  }
+  const Type_extra_attributes type_extra_attributes() const override
+  {
+    return *this;
+  }
 
   uint cols() const override;
   Item* element_index(uint i) override
@@ -375,7 +380,6 @@ class Item_exists_subselect :public Item_subselect
 protected:
   Item_func_not *upper_not;
   bool value; /* value of this item (boolean: exists/not-exists) */
-  bool abort_on_null;
 
   void init_length_and_dec();
   bool select_prepare_to_be_in();
@@ -399,7 +403,7 @@ public:
 
   Item_exists_subselect(THD *thd_arg, st_select_lex *select_lex);
   Item_exists_subselect(THD *thd_arg):
-    Item_subselect(thd_arg), upper_not(NULL), abort_on_null(0),
+  Item_subselect(thd_arg), upper_not(NULL),
     emb_on_expr_nest(NULL), optimizer(0), exists_transformed(0)
   {}
 
@@ -426,8 +430,6 @@ public:
   bool fix_length_and_dec() override;
   void print(String *str, enum_query_type query_type) override;
   bool select_transformer(JOIN *join) override;
-  void top_level_item() override { abort_on_null=1; }
-  bool is_top_level_item() const override { return abort_on_null; }
   bool exists2in_processor(void *opt_arg) override;
 
   Item* expr_cache_insert_transformer(THD *thd, uchar *unused) override;
@@ -832,8 +834,7 @@ public:
   enum enum_engine_type {ABSTRACT_ENGINE, SINGLE_SELECT_ENGINE,
                          UNION_ENGINE, UNIQUESUBQUERY_ENGINE,
                          INDEXSUBQUERY_ENGINE, HASH_SJ_ENGINE,
-                         ROWID_MERGE_ENGINE, TABLE_SCAN_ENGINE,
-                         SINGLE_COLUMN_ENGINE};
+                         ROWID_MERGE_ENGINE, TABLE_SCAN_ENGINE};
 
   subselect_engine(Item_subselect *si,
                    select_result_interceptor *res):
@@ -1176,9 +1177,6 @@ public:
     PARTIAL_MATCH,  /* Use some partial matching strategy. */
     PARTIAL_MATCH_MERGE, /* Use partial matching through index merging. */
     PARTIAL_MATCH_SCAN,  /* Use partial matching through table scan. */
-    SINGLE_COLUMN_MATCH, /* Use simplified matching when there is only
-                            one field involved. */
-    CONST_RETURN_NULL, /* The result of IN predicate is constant NULL */
     IMPOSSIBLE      /* Subquery materialization is not applicable. */
   };
 
@@ -1199,8 +1197,7 @@ protected:
   ulonglong rowid_merge_buff_size(bool has_non_null_key,
                                   bool has_covering_null_row,
                                   MY_BITMAP *partial_match_key_parts);
-  void choose_partial_match_strategy(uint field_count,
-                                     bool has_non_null_key,
+  void choose_partial_match_strategy(bool has_non_null_key,
                                      bool has_covering_null_row,
                                      MY_BITMAP *partial_match_key_parts);
   bool make_semi_join_conds();
@@ -1296,9 +1293,9 @@ protected:
     Quick sort comparison function that compares two rows of the same table
     indentfied with their row numbers.
   */
-  int cmp_keys_by_row_data(const rownum_t a, const rownum_t b) const;
-  static int cmp_keys_by_row_data_and_rownum(void *key, const void *a,
-                                             const void *b);
+  int cmp_keys_by_row_data(rownum_t a, rownum_t b);
+  static int cmp_keys_by_row_data_and_rownum(Ordered_key *key,
+                                             rownum_t* a, rownum_t* b);
 
   int cmp_key_with_search_key(rownum_t row_num);
 
@@ -1314,23 +1311,23 @@ public:
   /* Initialize a single-column index. */
   bool init(int col_idx);
 
-  uint get_column_count() const { return key_column_count; }
-  uint get_keyid() const { return keyid; }
-  Field *get_field(uint i) const
+  uint get_column_count() { return key_column_count; }
+  uint get_keyid() { return keyid; }
+  Field *get_field(uint i)
   {
     DBUG_ASSERT(i < key_column_count);
     return key_columns[i]->field;
   }
-  rownum_t get_min_null_row() const { return min_null_row; }
-  rownum_t get_max_null_row() const { return max_null_row; }
+  rownum_t get_min_null_row() { return min_null_row; }
+  rownum_t get_max_null_row() { return max_null_row; }
   MY_BITMAP * get_null_key() { return &null_key; }
-  ha_rows get_null_count() const { return null_count; }
-  ha_rows get_key_buff_elements() const { return key_buff_elements; }
+  ha_rows get_null_count() { return null_count; }
+  ha_rows get_key_buff_elements() { return key_buff_elements; }
   /*
     Get the search key element that corresponds to the i-th key part of this
     index.
   */
-  Item *get_search_key(uint i) const
+  Item *get_search_key(uint i)
   {
     return search_key->element_index(key_columns[i]->field->field_index);
   }
@@ -1343,7 +1340,7 @@ public:
   }
 
   bool sort_keys();
-  inline double null_selectivity() const;
+  double null_selectivity();
 
   /*
     Position the current element at the first row that matches the key.
@@ -1381,7 +1378,7 @@ public:
   {
     bitmap_set_bit(&null_key, (uint)row_num);
   }
-  bool is_null(rownum_t row_num) const
+  bool is_null(rownum_t row_num)
   {
     /*
       Indexes consisting of only NULLs do not have a bitmap buffer at all.
@@ -1397,7 +1394,7 @@ public:
       return FALSE;
     return bitmap_is_set(&null_key, (uint)row_num);
   }
-  void print(String *str) const;
+  void print(String *str);
 };
 
 
@@ -1510,18 +1507,20 @@ protected:
     Priority queue of Ordered_key indexes, one per NULLable column.
     This queue is used by the partial match algorithm in method exec().
   */
-  QUEUE pq;
+  Queue<Ordered_key> pq;
 protected:
   /*
     Comparison function to compare keys in order of decreasing bitmap
     selectivity.
   */
-  static int cmp_keys_by_null_selectivity(const void *k1, const void *k2);
+  static int cmp_keys_by_null_selectivity(Ordered_key **k1, Ordered_key **k2);
   /*
     Comparison function used by the priority queue pq, the 'smaller' key
     is the one with the smaller current row number.
   */
-  static int cmp_keys_by_cur_rownum(void *, const void *k1, const void *k2);
+  static int cmp_keys_by_cur_rownum(void *arg,
+                                    const Ordered_key *k1,
+                                    const Ordered_key *k2);
 
   bool test_null_row(rownum_t row_num);
   bool exists_complementing_null_row(MY_BITMAP *keys_to_complement);
@@ -1567,37 +1566,6 @@ public:
   enum_engine_type engine_type() override { return TABLE_SCAN_ENGINE; }
 };
 
-
-/*
-  An engine to handle NULL-aware Materialization for subqueries
-  that compare one column:
-
-    col1 IN (SELECT t2.col2 FROM t2 ...)
-
-  When only one column is used, we need to handle NULL values of
-  col1 and col2 but don't need to perform "partial" matches when only
-  a subset of compared columns is NULL.
-  This allows to save on some data structures.
-*/
-
-class subselect_single_column_match_engine:
-    public subselect_partial_match_engine
-{
-protected:
-  bool partial_match() override;
-public:
-  subselect_single_column_match_engine(THD *thd,
-                              subselect_uniquesubquery_engine *engine_arg,
-                              TABLE *tmp_table_arg, Item_subselect *item_arg,
-                              select_result_interceptor *result_arg,
-                              List<Item> *equi_join_conds_arg,
-                              bool has_covering_null_row_arg,
-                              bool has_covering_null_columns_arg,
-                              uint count_columns_with_nulls_arg);
-  void cleanup() override {}
-  enum_engine_type engine_type() override { return SINGLE_COLUMN_ENGINE; }
-};
-
 /**
   @brief Subquery materialization tracker
 
@@ -1680,10 +1648,6 @@ private:
         return "index_lookup;array merge for partial match";
       case Strategy::PARTIAL_MATCH_SCAN:
         return "index_lookup;full scan for partial match";
-      case Strategy::SINGLE_COLUMN_MATCH:
-        return "null-aware index_lookup";
-      case Strategy::CONST_RETURN_NULL:
-        return "return NULL";
       default:
         return "unsupported";
     }
