@@ -608,17 +608,13 @@ int Repl_semi_sync_master::report_reply_packet(uint32 server_id,
       DBUG_RETURN(-1);
     }
     else
-      sql_print_error("Read semi-sync reply magic number error. "
-                      "Got magic: %u  command %u  length: %lu",
-                      (uint) packet[REPLY_MAGIC_NUM_OFFSET], (uint) packet[0],
-                      packet_len);
+      sql_print_error("Read semi-sync reply magic number error");
     goto l_end;
   }
 
   if (unlikely(packet_len < REPLY_BINLOG_NAME_OFFSET))
   {
-    sql_print_error("Read semi-sync reply length error: packet is too small: %lu",
-                    packet_len);
+    sql_print_error("Read semi-sync reply length error: packet is too small");
     goto l_end;
   }
 
@@ -626,8 +622,7 @@ int Repl_semi_sync_master::report_reply_packet(uint32 server_id,
   log_file_len = packet_len - REPLY_BINLOG_NAME_OFFSET;
   if (unlikely(log_file_len >= FN_REFLEN))
   {
-    sql_print_error("Read semi-sync reply binlog file length too large: %llu",
-                    (ulonglong) log_file_pos);
+    sql_print_error("Read semi-sync reply binlog file length too large");
     goto l_end;
   }
   strncpy(log_file_name, (const char*)packet + REPLY_BINLOG_NAME_OFFSET, log_file_len);
@@ -646,7 +641,7 @@ int Repl_semi_sync_master::report_reply_packet(uint32 server_id,
 l_end:
   {
     char buf[256];
-    octet2hex(buf, (const char*) packet,
+    octet2hex(buf, (const unsigned char*) packet,
               MY_MIN(sizeof(buf)-1, (size_t) packet_len));
     sql_print_information("First bytes of the packet from semisync slave "
                           "server-id %d: %s", server_id, buf);
@@ -938,8 +933,8 @@ int Repl_semi_sync_master::commit_trx(const char *trx_wait_binlog_name,
             sql_print_information(
                 "Skipping semi-sync wait for transaction at pos %s, %lu. This "
                 "should be because semi-sync turned off and on during the "
-                "lifetime of this transaction.", trx_wait_binlog_name,
-                static_cast<unsigned long>(trx_wait_binlog_pos)););
+                "lifetime of this transaction.",
+                trx_wait_binlog_name, trx_wait_binlog_pos););
 
         /* The only known reason for a missing entry at this point is if
          * semi-sync was turned off then on, so on debug builds, we track
@@ -1451,6 +1446,22 @@ void Repl_semi_sync_master::set_export_stats()
   unlock();
 }
 
+void Repl_semi_sync_master::reset_stats()
+{
+  lock();
+  rpl_semi_sync_master_yes_transactions = 0;
+  rpl_semi_sync_master_no_transactions = 0;
+  rpl_semi_sync_master_off_times = 0;
+  rpl_semi_sync_master_timefunc_fails = 0;
+  rpl_semi_sync_master_wait_sessions = 0;
+  rpl_semi_sync_master_wait_pos_backtraverse = 0;
+  rpl_semi_sync_master_trx_wait_num = 0;
+  rpl_semi_sync_master_trx_wait_time = 0;
+  rpl_semi_sync_master_net_wait_num = 0;
+  rpl_semi_sync_master_net_wait_time = 0;
+  unlock();
+}
+
 void Repl_semi_sync_master::await_all_slave_replies(const char *msg)
 {
   struct timespec timeout;
@@ -1473,7 +1484,7 @@ void Repl_semi_sync_master::await_all_slave_replies(const char *msg)
     if (msg && first)
     {
       first= false;
-      sql_print_information("%s", msg);
+      sql_print_information(msg);
     }
 
     wait_result=
