@@ -60,6 +60,16 @@ void Session_sysvars_tracker::vars_list::copy(vars_list* from, THD *thd)
   from->init();
 }
 
+Session_sysvars_tracker::
+sysvar_node_st *Session_sysvars_tracker::vars_list::search(const sys_var *svar)
+{
+  return reinterpret_cast<sysvar_node_st*>(
+           my_hash_search(&m_registered_sysvars,
+                         reinterpret_cast<const uchar*>(&svar->offset),
+                         sizeof(svar->offset)));
+}
+
+
 /**
   Inserts the variable to be tracked into m_registered_sysvars hash.
 
@@ -507,6 +517,22 @@ bool Session_sysvars_tracker::store(THD *thd, String *buf)
   return false;
 }
 
+/* Parse all session track system variables if not parsed yet. */
+void Session_sysvars_tracker::maybe_parse_all(THD *thd)
+{
+  if (!m_parsed)
+  {
+    DBUG_ASSERT(thd->variables.session_track_system_variables);
+    LEX_STRING tmp= { thd->variables.session_track_system_variables,
+                      strlen(thd->variables.session_track_system_variables) };
+    if (orig_list.parse_var_list(thd, tmp, true, thd->charset()))
+    {
+      orig_list.reinit();
+      return;
+    }
+    m_parsed= true;
+  }
+}
 
 /**
   Mark the system variable as changed.
@@ -521,18 +547,7 @@ void Session_sysvars_tracker::mark_as_changed(THD *thd, const sys_var *var)
   if (!is_enabled())
     return;
 
-  if (!m_parsed)
-  {
-    DBUG_ASSERT(thd->variables.session_track_system_variables);
-    LEX_STRING tmp= { thd->variables.session_track_system_variables,
-                      strlen(thd->variables.session_track_system_variables) };
-    if (orig_list.parse_var_list(thd, tmp, true, thd->charset()))
-    {
-      orig_list.reinit();
-      return;
-    }
-    m_parsed= true;
-  }
+  maybe_parse_all(thd);
 
   /*
     Check if the specified system variable is being tracked, if so
@@ -545,6 +560,23 @@ void Session_sysvars_tracker::mark_as_changed(THD *thd, const sys_var *var)
   }
 }
 
+/**
+  Mark all session tracking system variables as changed.
+*/
+void Session_sysvars_tracker::mark_all_as_changed(THD *thd)
+{
+  if (!is_enabled())
+    return;
+
+  maybe_parse_all(thd);
+
+  for (ulong i= 0; i < orig_list.size(); i++)
+  {
+    orig_list.at(i)->m_changed= true;
+    set_changed(thd);
+  }
+}
+
 
 /**
   Supply key to a hash.
@@ -560,8 +592,9 @@ uchar *Session_sysvars_tracker::sysvars_get_key(const char *entry,
                                                 size_t *length,
                                                 my_bool not_used __attribute__((unused)))
 {
-  *length= sizeof(sys_var *);
-  return (uchar *) &(((sysvar_node_st *) entry)->m_svar);
+  auto key=&(((sysvar_node_st *) entry)->m_svar->offset);
+  *length= sizeof(*key);
+  return (uchar *) key;
 }
 
 
