@@ -134,12 +134,6 @@ public:
     return sph ? sph->sp_handler_mysql_proc() : NULL;
   }
 
-  static bool eq_routine_name(const LEX_CSTRING &name1,
-                              const LEX_CSTRING &name2)
-  {
-    return system_charset_info->strnncoll(name1.str, name1.length,
-                                          name2.str, name2.length) == 0;
-  }
   const char *type_str() const { return type_lex_cstring().str; }
   virtual const char *show_create_routine_col1_caption() const
   {
@@ -157,6 +151,8 @@ public:
   }
   virtual enum_sp_type type() const= 0;
   virtual LEX_CSTRING type_lex_cstring() const= 0;
+  virtual enum_sql_command sqlcom_create() const= 0;
+  virtual enum_sql_command sqlcom_drop() const= 0;
   virtual LEX_CSTRING empty_body_lex_cstring(sql_mode_t mode) const
   {
     static LEX_CSTRING m_empty_body= {STRING_WITH_LEN("???")};
@@ -256,6 +252,14 @@ public:
     static LEX_CSTRING m_type_str= { STRING_WITH_LEN("PROCEDURE")};
     return m_type_str;
   }
+  enum_sql_command sqlcom_create() const override
+  {
+    return SQLCOM_CREATE_PROCEDURE;
+  }
+  enum_sql_command sqlcom_drop() const override
+  {
+    return SQLCOM_DROP_PROCEDURE;
+  }
   LEX_CSTRING empty_body_lex_cstring(sql_mode_t mode) const override;
   const char *show_create_routine_col1_caption() const override
   {
@@ -306,6 +310,14 @@ public:
     static LEX_CSTRING m_type_str= { STRING_WITH_LEN("FUNCTION")};
     return m_type_str;
   }
+  enum_sql_command sqlcom_create() const override
+  {
+    return SQLCOM_CREATE_FUNCTION;
+  }
+  enum_sql_command sqlcom_drop() const override
+  {
+    return SQLCOM_DROP_FUNCTION;
+  }
   LEX_CSTRING empty_body_lex_cstring(sql_mode_t mode) const override;
   const char *show_create_routine_col1_caption() const override
   {
@@ -375,6 +387,14 @@ public:
     static LEX_CSTRING m_type_str= {STRING_WITH_LEN("PACKAGE")};
     return m_type_str;
   }
+  enum_sql_command sqlcom_create() const override
+  {
+    return SQLCOM_CREATE_PACKAGE;
+  }
+  enum_sql_command sqlcom_drop() const override
+  {
+    return SQLCOM_DROP_PACKAGE;
+  }
   LEX_CSTRING empty_body_lex_cstring(sql_mode_t mode) const override
   {
     static LEX_CSTRING m_empty_body= {STRING_WITH_LEN("BEGIN END")};
@@ -408,6 +428,14 @@ public:
     static LEX_CSTRING m_type_str= {STRING_WITH_LEN("PACKAGE BODY")};
     return m_type_str;
   }
+  enum_sql_command sqlcom_create() const override
+  {
+    return SQLCOM_CREATE_PACKAGE_BODY;
+  }
+  enum_sql_command sqlcom_drop() const override
+  {
+    return SQLCOM_DROP_PACKAGE_BODY;
+  }
   LEX_CSTRING empty_body_lex_cstring(sql_mode_t mode) const override
   {
     static LEX_CSTRING m_empty_body= {STRING_WITH_LEN("BEGIN END")};
@@ -441,6 +469,14 @@ public:
     static LEX_CSTRING m_type_str= { STRING_WITH_LEN("TRIGGER")};
     return m_type_str;
   }
+  enum_sql_command sqlcom_create() const override
+  {
+    return SQLCOM_CREATE_TRIGGER;
+  }
+  enum_sql_command sqlcom_drop() const override
+  {
+    return SQLCOM_DROP_TRIGGER;
+  }
   MDL_key::enum_mdl_namespace get_mdl_type() const override
   {
     DBUG_ASSERT(0);
@@ -582,7 +618,7 @@ enum
 
 /* Drop all routines in database 'db' */
 int
-sp_drop_db_routines(THD *thd, const char *db);
+sp_drop_db_routines(THD *thd, const LEX_CSTRING &db);
 
 /**
    Acquires exclusive metadata lock on all stored routines in the
@@ -594,7 +630,7 @@ sp_drop_db_routines(THD *thd, const char *db);
    @retval  false  Success
    @retval  true   Failure
  */
-bool lock_db_routines(THD *thd, const char *db);
+bool lock_db_routines(THD *thd, const Lex_ident_db_normalized &db);
 
 /**
   Structure that represents element in the set of stored routines
@@ -669,8 +705,4 @@ bool load_collation(THD *thd,MEM_ROOT *mem_root,
                     CHARSET_INFO *dflt_cl,
                     CHARSET_INFO **cl);
 
-void sp_returns_type(THD *thd,
-                     String &result,
-                     const sp_head *sp);
-
 #endif /* _SP_H_ */
