@@ -1,3 +1,20 @@
+/*
+   Copyright (c) 2002, 2016, Oracle and/or its affiliates.
+   Copyright (c) 2011, 2024, MariaDB
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+
 #include "sp_instr.h"
 
 #include "opt_trace.h"    // class Opt_trace_start
@@ -949,12 +966,13 @@ sp_instr_stmt::execute(THD *thd, uint *nextp)
     if (query_cache_send_result_to_client(thd, thd->query(),
                                           thd->query_length()) <= 0)
     {
-      thd->reset_slow_query_state();
+      thd->reset_slow_query_state(&backup_state);
       res= m_lex_keeper.validate_lex_and_exec_core(thd, nextp, false, this);
       bool log_slow= !res && thd->enable_slow_log;
 
       /* Finalize server status flags after executing a statement. */
-      if (log_slow || thd->get_stmt_da()->is_eof())
+      if (log_slow || thd->get_stmt_da()->is_eof() ||
+          mysql_audit_general_enabled())
         thd->update_server_status();
 
       if (thd->get_stmt_da()->is_eof())
@@ -2030,6 +2048,8 @@ sp_instr_cursor_copy_struct::exec_core(THD *thd, uint *nextp)
   int ret= 0;
   Item_field_row *row= (Item_field_row*) thd->spcont->get_variable(m_var);
   DBUG_ASSERT(row->type_handler() == &type_handler_row);
+  DBUG_ASSERT(row->field);
+  DBUG_ASSERT(dynamic_cast<Field_row*>(row->field));
 
   /*
     Copy structure only once. If the cursor%ROWTYPE variable is declared
@@ -2052,13 +2072,17 @@ sp_instr_cursor_copy_struct::exec_core(THD *thd, uint *nextp)
         where explicit ROW elements and table%ROWTYPE reside:
         - tmp.export_structure() allocates new Spvar_definition instances
           and their components (such as TYPELIBs).
-        - row->row_create_items() creates new Item_field instances.
+        - field->row_create_fields() creates a new Virtual_tmp_table instance
+          with Field instances, one Field instance per a ROW member.
+        - row->add_array_of_item_field() creates Item_field instances
+          corresponding to Field instances.
         They all are created on the same mem_root.
       */
       Query_arena current_arena;
       thd->set_n_backup_active_arena(thd->spcont->callers_arena, &current_arena);
-      if (!(ret= tmp.export_structure(thd, &defs)))
-        row->row_create_items(thd, &defs);
+      ret= tmp.export_structure(thd, &defs) ||
+           static_cast<Field_row*>(row->field)->row_create_fields(thd, &defs) ||
+           row->add_array_of_item_field(thd, *row->field->virtual_tmp_table());
       thd->restore_active_arena(thd->spcont->callers_arena, &current_arena);
       tmp.close(thd);
     }
