@@ -28,6 +28,7 @@ struct Name_resolution_context;
 class Open_table_context;
 class Open_tables_state;
 class Prelocking_strategy;
+class DML_prelocking_strategy;
 struct TABLE_LIST;
 class THD;
 struct handlerton;
@@ -179,7 +180,8 @@ bool fill_record_n_invoke_before_triggers(THD *thd, TABLE *table,
                                           bool ignore_errors,
                                           enum trg_event_type event);
 bool insert_fields(THD *thd, Name_resolution_context *context,
-		   const char *db_name, const char *table_name,
+                   const Lex_ident_db &db_name,
+                   const Lex_ident_table &table_name,
                    List_iterator<Item> *it, bool any_privileges,
                    uint *hidden_bit_fields, bool returning_field);
 void make_leaves_list(THD *thd, List<TABLE_LIST> &list, TABLE_LIST *tables,
@@ -206,7 +208,7 @@ find_field_in_tables(THD *thd, Item_ident *item,
                      bool check_privileges, bool register_tree_change);
 Field *
 find_field_in_table_ref(THD *thd, TABLE_LIST *table_list,
-                        const char *name, size_t length,
+                        const Lex_ident_column &name,
                         const char *item_name, const char *db_name,
                         const char *table_name,
                         ignored_tables_list_t ignored_tables,
@@ -214,10 +216,10 @@ find_field_in_table_ref(THD *thd, TABLE_LIST *table_list,
                         field_index_t *cached_field_index_ptr,
                         bool register_tree_change, TABLE_LIST **actual_table);
 Field *
-find_field_in_table(THD *thd, TABLE *table, const char *name, size_t length,
+find_field_in_table(THD *thd, TABLE *table, const Lex_ident_column &name,
                     bool allow_rowid, field_index_t *cached_field_index_ptr);
 Field *
-find_field_in_table_sef(TABLE *table, const char *name);
+find_field_in_table_sef(TABLE *table, const Lex_ident_column &name);
 Item ** find_item_in_list(Item *item, List<Item> &items, uint *counter,
                           find_item_error_report_type report_error,
                           enum_resolution_type *resolution, uint limit= 0);
@@ -293,6 +295,9 @@ bool open_normal_and_derived_tables(THD *thd, TABLE_LIST *tables, uint flags,
 bool open_tables_only_view_structure(THD *thd, TABLE_LIST *tables,
                                      bool can_deadlock);
 bool open_and_lock_internal_tables(TABLE *table, bool lock);
+bool open_tables_for_query(THD *thd, TABLE_LIST *tables,
+                           uint *table_count, uint flags,
+                           DML_prelocking_strategy *prelocking_strategy);
 bool lock_tables(THD *thd, TABLE_LIST *tables, uint counter, uint flags);
 int decide_logging_format(THD *thd, TABLE_LIST *tables);
 void close_thread_table(THD *thd, TABLE **table_ptr);
@@ -363,14 +368,14 @@ inline void setup_table_map(TABLE *table, TABLE_LIST *table_list, uint tablenr)
   }
   table->tablenr= tablenr;
   table->map= (table_map) 1 << tablenr;
-  table->force_index= table_list->force_index;
+  table->force_index= table->force_index_join= 0;
   table->force_index_order= table->force_index_group= 0;
   table->covering_keys= table->s->keys_for_keyread;
 }
 
 inline TABLE_LIST *find_table_in_global_list(TABLE_LIST *table,
-                                             LEX_CSTRING *db_name,
-                                             LEX_CSTRING *table_name)
+                                             const LEX_CSTRING *db_name,
+                                             const LEX_CSTRING *table_name)
 {
   return find_table_in_list(table, &TABLE_LIST::next_global,
                             db_name, table_name);
@@ -437,6 +442,17 @@ public:
 };
 
 
+
+class Multiupdate_prelocking_strategy : public DML_prelocking_strategy
+{
+  bool done;
+  bool has_prelocking_list;
+public:
+  void reset(THD *thd) override;
+  bool handle_end(THD *thd) override;
+};
+
+
 /**
   A strategy for prelocking algorithm to be used for LOCK TABLES
   statement.
