@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2016, 2021, MariaDB
+   Copyright (c) 2016, 2017 MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include "sql_view.h"    // for make_valid_column_names
 #include "sql_parse.h"
 #include "sql_select.h"
+#include "sql_show.h"                // append_definer, append_identifier
 
 
 /**
@@ -55,6 +56,14 @@ bool With_clause::add_with_element(With_element *elem)
 }
 
 
+void  st_select_lex_unit::set_with_clause(With_clause *with_cl)
+{ 
+  with_clause= with_cl;
+  if (with_clause)
+    with_clause->set_owner(this);
+}
+
+
 /**
   @brief
     Check dependencies between tables defined in a list of with clauses
@@ -91,55 +100,13 @@ bool LEX::check_dependencies_in_with_clauses()
 }
 
 
-/**
-  @brief
-    Resolve references to CTE in specification of hanging CTE
-
-  @details
-    A CTE to which there are no references in the query is called hanging CTE.
-    Although such CTE is not used for execution its specification must be
-    subject to context analysis. All errors concerning references to
-    non-existing tables or fields occurred in the specification must be
-    reported as well as all other errors caught at the prepare stage.
-    The specification of a hanging CTE might contain references to other
-    CTE outside of the specification and within it if the specification
-    contains a with clause. This function resolves all such references for
-    all hanging CTEs encountered in the processed query.
-
-  @retval
-    false   on success
-    true    on failure
-*/
-
-bool
-LEX::resolve_references_to_cte_in_hanging_cte()
-{
-  for (With_clause *with_clause= with_clauses_list;
-       with_clause; with_clause= with_clause->next_with_clause)
-  {
-    for (With_element *with_elem= with_clause->with_list.first;
-         with_elem; with_elem= with_elem->next)
-    {
-      if (!with_elem->is_referenced())
-      {
-        TABLE_LIST *first_tbl=
-                     with_elem->spec->first_select()->table_list.first;
-        TABLE_LIST **with_elem_end_pos= with_elem->head->tables_pos.end_pos;
-        if (first_tbl && resolve_references_to_cte(first_tbl, with_elem_end_pos))
-          return true;
-      }
-    }
-  }
-  return false;
-}
-
-
 /**
   @brief
     Resolve table references to CTE from a sub-chain of table references
 
   @param tables      Points to the beginning of the sub-chain
   @param tables_last Points to the address with the sub-chain barrier
+  @param excl_spec   Ignore the definition with this spec
 
   @details
     The method resolves tables references to CTE from the chain of
@@ -181,7 +148,8 @@ LEX::resolve_references_to_cte_in_hanging_cte()
 */
 
 bool LEX::resolve_references_to_cte(TABLE_LIST *tables,
-                                    TABLE_LIST **tables_last)
+                                    TABLE_LIST **tables_last,
+                                    st_select_lex_unit *excl_spec)
 {
   With_element *with_elem= 0;
 
@@ -189,15 +157,16 @@ bool LEX::resolve_references_to_cte(TABLE_LIST *tables,
   {
     if (tbl->derived)
       continue;
-    if (!tbl->db.length && !tbl->with)
-      tbl->with= tbl->select_lex->find_table_def_in_with_clauses(tbl);
+    if (!tbl->db.str && !tbl->with)
+      tbl->with= tbl->select_lex->find_table_def_in_with_clauses(tbl,
+                                                                 excl_spec);
     if (!tbl->with)    // no CTE matches table reference tbl
     {
       if (only_cte_resolution)
         continue;
-      if (!tbl->db.length)   // no database specified in table reference tbl
+      if (!tbl->db.str)   // no database specified in table reference tbl
       {
-        if (!thd->db.length) // no default database is set
+        if (!thd->db.str) // no default database is set
         {
           my_message(ER_NO_DB_ERROR, ER(ER_NO_DB_ERROR), MYF(0));
           return true;
@@ -205,9 +174,9 @@ bool LEX::resolve_references_to_cte(TABLE_LIST *tables,
         if (copy_db_to(&tbl->db))
           return true;
         if (!(tbl->table_options & TL_OPTION_ALIAS))
-          tbl->mdl_request.init(MDL_key::TABLE, tbl->db.str,
-                                tbl->table_name.str,
-                                tbl->mdl_type, MDL_TRANSACTION);
+          MDL_REQUEST_INIT(&tbl->mdl_request, MDL_key::TABLE,
+                           tbl->db.str, tbl->table_name.str,
+                           tbl->mdl_type, MDL_TRANSACTION);
         tbl->mdl_request.set_type((tbl->lock_type >= TL_WRITE_ALLOW_WRITE) ?
                                    MDL_SHARED_WRITE : MDL_SHARED_READ);
       }
@@ -243,7 +212,8 @@ bool LEX::resolve_references_to_cte(TABLE_LIST *tables,
     tbl->schema_table= 0;
     if (tbl->derived)
     {
-      tbl->derived->first_select()->linkage= DERIVED_TABLE_TYPE;
+      tbl->derived->first_select()->set_linkage(DERIVED_TABLE_TYPE);
+      tbl->select_lex->add_statistics(tbl->derived);
     }
     if (tbl->with->is_recursive && tbl->is_with_table_recursive_reference())
       continue;
@@ -277,9 +247,7 @@ LEX::check_cte_dependencies_and_resolve_references()
     return true;
   if (!with_cte_resolution)
     return false;
-  if (resolve_references_to_cte(query_tables, query_tables_last))
-    return true;
-  if (resolve_references_to_cte_in_hanging_cte())
+  if (resolve_references_to_cte(query_tables, query_tables_last, NULL))
     return true;
   return false;
 }
@@ -323,8 +291,7 @@ bool With_clause::check_dependencies()
          elem != with_elem;
          elem= elem->next)
     {
-      if (my_strcasecmp(system_charset_info, with_elem->get_name_str(),
-                        elem->get_name_str()) == 0)
+      if (with_elem->get_name().streq(elem->get_name()))
       {
         my_error(ER_DUP_QUERY_NAME, MYF(0),
                  with_elem->get_name_str());
@@ -423,6 +390,7 @@ bool With_element::check_dependencies_in_spec()
  
   @param table    The reference to the table that is looked for
   @param barrier  The barrier with element for the search
+  @param excl_spec Ignore the definition with this spec
 
   @details
     The function looks through the elements of this with clause trying to find
@@ -436,14 +404,16 @@ bool With_element::check_dependencies_in_spec()
 */    
 
 With_element *With_clause::find_table_def(TABLE_LIST *table,
-                                          With_element *barrier)
+                                          With_element *barrier,
+                                          st_select_lex_unit *excl_spec)
 {
   for (With_element *with_elem= with_list.first; 
        with_elem != barrier;
        with_elem= with_elem->next)
   {
-    if (my_strcasecmp(system_charset_info, with_elem->get_name_str(),
-		      table->table_name.str) == 0 &&
+    if (excl_spec && with_elem->spec == excl_spec)
+      continue;
+    if (with_elem->get_name().streq(table->table_name) &&
         !table->is_fqtn)
     {
       table->set_derived();
@@ -479,47 +449,33 @@ With_element *find_table_def_in_with_clauses(TABLE_LIST *tbl,
                                              st_unit_ctxt_elem *ctxt)
 {
   With_element *found= 0;
+  st_select_lex_unit *top_unit= 0;
   for (st_unit_ctxt_elem *unit_ctxt_elem= ctxt;
        unit_ctxt_elem;
        unit_ctxt_elem= unit_ctxt_elem->prev)
   {
     st_select_lex_unit *unit= unit_ctxt_elem->unit;
     With_clause *with_clause= unit->with_clause;
-    /*
-      First look for the table definition in the with clause attached to 'unit'
-      if there is any such clause.
-    */
     if (with_clause)
     {
-      found= with_clause->find_table_def(tbl, NULL);
+      /*
+        If the reference to tbl that has to be resolved belongs to
+        the FROM clause of a descendant of top_unit->with_element
+        and this with element belongs to with_clause then this
+        element must be used as the barrier for the search in the
+        the list of CTEs from with_clause unless the clause contains
+        RECURSIVE.
+      */
+      With_element *barrier= 0;
+      if (top_unit && !with_clause->with_recursive &&
+          top_unit->with_element &&
+          top_unit->with_element->get_owner() == with_clause)
+        barrier= top_unit->with_element;
+      found= with_clause->find_table_def(tbl, barrier, NULL);
       if (found)
         break;
     }
-    /*
-      If 'unit' is the unit that defines a with element then reset 'unit'
-      to the unit whose attached with clause contains this with element.
-    */
-    With_element *with_elem= unit->with_element;
-    if (with_elem)
-    {
-      if (!(unit_ctxt_elem= unit_ctxt_elem->prev))
-        break;
-      unit= unit_ctxt_elem->unit;
-    }
-    with_clause= unit->with_clause;
-    /*
-      Now look for the table definition in this with clause. If the with clause
-      contains RECURSIVE the search is performed through all CTE definitions in
-      clause, otherwise up to the definition of 'with_elem' unless it is NULL.
-    */
-    if (with_clause)
-    {
-      found= with_clause->find_table_def(tbl,
-                                         with_clause->with_recursive ?
-			                 NULL : with_elem);
-      if (found)
-	break;
-    }
+    top_unit= unit;
   }
   return found;
 }
@@ -570,10 +526,11 @@ void With_element::check_dependencies_in_select(st_select_lex *sl,
     {
       With_clause *with_clause= sl->master_unit()->with_clause;
       if (with_clause)
-        tbl->with= with_clause->find_table_def(tbl, NULL);
+        tbl->with= with_clause->find_table_def(tbl, NULL, NULL);
       if (!tbl->with)
         tbl->with= owner->find_table_def(tbl,
-                                         owner->with_recursive ? NULL : this);
+                                         owner->with_recursive ? NULL : this,
+                                         NULL);
     }
     if (!tbl->with)
       tbl->with= find_table_def_in_with_clauses(tbl, ctxt);
@@ -692,6 +649,8 @@ void With_element::check_dependencies_in_unit(st_select_lex_unit *unit,
   {
     check_dependencies_in_select(sl, &unit_ctxt_elem, in_subq, dep_map);
   }
+  if ((sl= unit->fake_select_lex))
+    check_dependencies_in_select(sl, &unit_ctxt_elem, in_subq, dep_map);
 }
 
 
@@ -918,7 +877,7 @@ void With_element::move_anchors_ahead()
 {
   st_select_lex *next_sl;
   st_select_lex *new_pos= spec->first_select();
-  new_pos->linkage= UNION_TYPE;
+  new_pos->set_linkage(UNION_TYPE);
   for (st_select_lex *sl= new_pos; sl; sl= next_sl)
   {
     next_sl= sl->next_select(); 
@@ -927,9 +886,9 @@ void With_element::move_anchors_ahead()
       sl->move_node(new_pos);
       if (new_pos == spec->first_select())
       {
-        enum sub_select_type type= new_pos->linkage;
-        new_pos->linkage= sl->linkage;
-        sl->linkage= type;
+        enum sub_select_type type= new_pos->get_linkage();
+        new_pos->set_linkage(sl->get_linkage());
+        sl->set_linkage(type);
         new_pos->with_all_modifier= sl->with_all_modifier;
         sl->with_all_modifier= false;
       }
@@ -942,7 +901,7 @@ void With_element::move_anchors_ahead()
     }
   }
   first_recursive= new_pos;
-  spec->first_select()->linkage= DERIVED_TABLE_TYPE;
+  spec->first_select()->set_linkage(DERIVED_TABLE_TYPE);
 }
 
 
@@ -996,7 +955,9 @@ bool With_clause::prepare_unreferenced_elements(THD *thd)
     true    on failure
 */
   
-bool With_element::set_unparsed_spec(THD *thd, char *spec_start, char *spec_end,
+bool With_element::set_unparsed_spec(THD *thd,
+                                     const char *spec_start,
+                                     const char *spec_end,
                                      my_ptrdiff_t spec_offset)
 {
   stmt_prepare_mode= thd->m_parser_state->m_lip.stmt_prepare_mode;
@@ -1010,7 +971,7 @@ bool With_element::set_unparsed_spec(THD *thd, char *spec_start, char *spec_end,
 
   if (!unparsed_spec.str)
   {
-    my_error(ER_OUTOFMEMORY, MYF(ME_FATALERROR), 
+    my_error(ER_OUTOFMEMORY, MYF(ME_FATAL), 
              static_cast<int>(unparsed_spec.length));
     return true;
   }
@@ -1071,7 +1032,7 @@ st_select_lex_unit *With_element::clone_parsed_spec(LEX *old_lex,
   st_select_lex *last_clone_select;
 
   char save_end= unparsed_spec.str[unparsed_spec.length];
-  const_cast<char*>(unparsed_spec.str)[unparsed_spec.length]= '\0';
+  ((char*) &unparsed_spec.str[unparsed_spec.length])[0]= '\0';
 
   lex_start(thd);
   lex->clone_spec_offset= unparsed_spec_offset;
@@ -1123,9 +1084,8 @@ st_select_lex_unit *With_element::clone_parsed_spec(LEX *old_lex,
                                 (unsigned int)unparsed_spec.length,
                                 stmt_prepare_mode);
 
-  const_cast<char*>(unparsed_spec.str)[unparsed_spec.length]= save_end;
+  ((char*) &unparsed_spec.str[unparsed_spec.length])[0]= save_end;
   with_select= lex->unit.first_select();
-  with_select->select_number= ++lex->stmt_lex->current_select_number;
 
   if (parse_status)
     goto err;
@@ -1147,7 +1107,8 @@ st_select_lex_unit *With_element::clone_parsed_spec(LEX *old_lex,
   */
   lex->only_cte_resolution= old_lex->only_cte_resolution;
   if (lex->resolve_references_to_cte(lex->query_tables,
-                                     lex->query_tables_last))
+                                     lex->query_tables_last,
+                                     spec))
   {
     res= NULL;
     goto err;
@@ -1219,9 +1180,9 @@ err:
     false  otherwise
 */    
 
-bool 
-With_element::rename_columns_of_derived_unit(THD *thd, 
-                                             st_select_lex_unit *unit)
+bool
+With_element::process_columns_of_derived_unit(THD *thd,
+                                              st_select_lex_unit *unit)
 {
   if (unit->columns_are_renamed)
     return false;
@@ -1231,7 +1192,7 @@ With_element::rename_columns_of_derived_unit(THD *thd,
   if (column_list.elements)  //  The column list is optional
   {
     List_iterator_fast<Item> it(select->item_list);
-    List_iterator_fast<LEX_CSTRING> nm(column_list);
+    List_iterator_fast<Lex_ident_sys> nm(column_list);
     Item *item;
     LEX_CSTRING *name;
 
@@ -1247,8 +1208,8 @@ With_element::rename_columns_of_derived_unit(THD *thd,
     /* Rename the columns of the first select in the unit */
     while ((item= it++, name= nm++))
     {
-      item->set_name(thd, name->str, (uint) name->length, system_charset_info);
-      item->is_autogenerated_name= false;
+      lex_string_set(&item->name, name->str);
+      item->base_flags|= item_base_t::IS_EXPLICIT_NAME;
     }
 
     if (arena)
@@ -1257,6 +1218,43 @@ With_element::rename_columns_of_derived_unit(THD *thd,
   else
     make_valid_column_names(thd, select->item_list);
 
+  if (cycle_list)
+  {
+    List_iterator_fast<Item> it(select->item_list);
+    List_iterator_fast<Lex_ident_sys> nm(*cycle_list);
+    List_iterator_fast<Lex_ident_sys> nm_check(*cycle_list);
+    DBUG_ASSERT(cycle_list->elements != 0);
+    while (LEX_CSTRING *name= nm++)
+    {
+      Item *item;
+      /*
+        Check for uniqueness of each element in the cycle list:
+        It's sufficient to check that there is no duplicate of 'name'
+        among the elements that precede it.
+      */
+      LEX_CSTRING *check;
+      nm_check.rewind();
+      while ((check= nm_check++) && check != name)
+      {
+        if (check->length == name->length &&
+            strncmp(check->str, name->str, name->length) == 0)
+        {
+          my_error(ER_DUP_FIELDNAME, MYF(0), check->str);
+          return true;
+        }
+      }
+      /* Check that 'name' is the name of a column of the processed CTE */
+      while ((item= it++) &&
+             (item->name.length != name->length ||
+              strncmp(item->name.str, name->str, name->length) != 0));
+      if (item == NULL)
+      {
+        my_error(ER_BAD_FIELD_ERROR, MYF(0), name->str, "CYCLE clause");
+        return true;
+      }
+      item->base_flags|= item_base_t::IS_IN_WITH_CYCLE;
+    }
+  }
   unit->columns_are_renamed= true;
 
   return false;
@@ -1290,14 +1288,14 @@ bool With_element::prepare_unreferenced(THD *thd)
        sl= sl->next_select())
     sl->context.outer_context= 0;
 
+  uint8 save_context_analysys_only= thd->lex->context_analysis_only;
   thd->lex->context_analysis_only|= CONTEXT_ANALYSIS_ONLY_DERIVED;
   if (!spec->prepared &&
       (spec->prepare(spec->derived, 0, 0) ||
-       rename_columns_of_derived_unit(thd, spec) ||
+       process_columns_of_derived_unit(thd, spec) ||
        check_duplicate_names(thd, first_sl->item_list, 1)))
     rc= true;
- 
-  thd->lex->context_analysis_only&= ~CONTEXT_ANALYSIS_ONLY_DERIVED;
+  thd->lex->context_analysis_only= save_context_analysys_only;
   return rc;
 }
 
@@ -1313,6 +1311,7 @@ bool With_element::is_anchor(st_select_lex *sel)
      Search for the definition of the given table referred in this select node
 
   @param table  reference to the table whose definition is searched for
+  @param excl_spec  ignore the definition with this spec
      
   @details  
     The method looks for the definition of the table whose reference is occurred
@@ -1325,7 +1324,8 @@ bool With_element::is_anchor(st_select_lex *sel)
     NULL -  otherwise
 */    
 
-With_element *st_select_lex::find_table_def_in_with_clauses(TABLE_LIST *table)
+With_element *st_select_lex::find_table_def_in_with_clauses(TABLE_LIST *table,
+                                                 st_select_lex_unit *excl_spec)
 {
   With_element *found= NULL;
   With_clause *containing_with_clause= NULL;
@@ -1342,7 +1342,7 @@ With_element *st_select_lex::find_table_def_in_with_clauses(TABLE_LIST *table)
     With_clause *attached_with_clause= sl->get_with_clause();
     if (attached_with_clause &&
         attached_with_clause != containing_with_clause &&
-        (found= attached_with_clause->find_table_def(table, NULL)))
+        (found= attached_with_clause->find_table_def(table, NULL, excl_spec)))
       break;
     master_unit= sl->master_unit();
     outer_sl= master_unit->outer_select();
@@ -1352,7 +1352,8 @@ With_element *st_select_lex::find_table_def_in_with_clauses(TABLE_LIST *table)
       containing_with_clause= with_elem->get_owner();
       With_element *barrier= containing_with_clause->with_recursive ?
                                NULL : with_elem;
-      if ((found= containing_with_clause->find_table_def(table, barrier)))
+      if ((found= containing_with_clause->find_table_def(table, barrier,
+                                                         excl_spec)))
         break;
       if (outer_sl && !outer_sl->get_with_element())
         break;
@@ -1619,16 +1620,17 @@ bool st_select_lex::check_subqueries_with_recursive_references()
 /**
   @brief
     Print this with clause
-  
-  @param str         Where to print to
-  @param query_type  The mode of printing 
-     
-  @details
-    The method prints a string representation of this clause in the 
-    string str. The parameter query_type specifies the mode of printing.
-*/    
 
-void With_clause::print(String *str, enum_query_type query_type)
+  @param thd         Thread handle
+  @param str         Where to print to
+  @param query_type  The mode of printing
+
+  @details
+    The method prints a string representation of this clause in the
+    string str. The parameter query_type specifies the mode of printing.
+*/
+
+void With_clause::print(THD *thd, String *str, enum_query_type query_type)
 {
   /*
     Any with clause contains just definitions of CTE tables.
@@ -1644,47 +1646,62 @@ void With_clause::print(String *str, enum_query_type query_type)
        with_elem= with_elem->next)
   {
     if (with_elem != with_list.first)
-      str->append(", ");
-    with_elem->print(str, query_type);
+      str->append(STRING_WITH_LEN(", "));
+    with_elem->print(thd, str, query_type);
   }
 }
 
 
+void list_strlex_print(THD *thd, String *str, List<Lex_ident_sys> *list,
+                              bool bracketed)
+{
+  List_iterator_fast<Lex_ident_sys> li(*list);
+  bool first= TRUE;
+
+  if (bracketed)
+    str->append('(');
+  while(Lex_ident_sys *col_name= li++)
+  {
+    if (first)
+      first= FALSE;
+    else
+      str->append(',');
+    append_identifier(thd, str, col_name);
+  }
+  if (bracketed)
+    str->append(')');
+}
+
+
 /**
    @brief
      Print this with element
-  
+
+  @param thd         Thread handle
   @param str         Where to print to
-  @param query_type  The mode of printing 
-     
+  @param query_type  The mode of printing
+
   @details
-    The method prints a string representation of this with element in the 
+    The method prints a string representation of this with element in the
     string str. The parameter query_type specifies the mode of printing.
 */
 
-void With_element::print(String *str, enum_query_type query_type)
+void With_element::print(THD *thd, String *str, enum_query_type query_type)
 {
   str->append(get_name());
   if (column_list.elements)
-  {
-    List_iterator_fast<LEX_CSTRING> li(column_list);
-    str->append('(');
-    for (LEX_CSTRING *col_name= li++; ; )
-    {
-      str->append(col_name);
-      col_name= li++;
-      if (!col_name)
-      {
-        str->append(')');
-        break;
-      }
-      str->append(',');
-    }
-  }
-  str->append(STRING_WITH_LEN(" as "));
-  str->append('(');
+    list_strlex_print(thd, str, &column_list, true);
+  str->append(STRING_WITH_LEN(" as ("));
   spec->print(str, query_type);
   str->append(')');
+
+  if (cycle_list)
+  {
+    DBUG_ASSERT(cycle_list->elements != 0);
+    str->append(STRING_WITH_LEN(" CYCLE "));
+    list_strlex_print(thd, str, cycle_list);
+    str->append(STRING_WITH_LEN(" RESTRICT "));
+  }
 }
 
 
@@ -1709,3 +1726,26 @@ bool With_element::instantiate_tmp_tables()
   return false;
 }
 
+void With_element::set_cycle_list(List<Lex_ident_sys> *cycle_list_arg)
+{
+  cycle_list= cycle_list_arg;
+
+  /*
+    If a CTE table with columns c1,...,cn is defined with a cycle
+    clause CYCLE(ci1,...,cik) then no two rows r1 and r2 from the
+    table shall have r1.ci1=r2.ci1 && ... && r1.cik=r2.cik.
+
+    If a cycle clause is used in the specification of a CTE then
+    each UNION ALL at the top level of the specification is interpreted
+    as a UNION DISTINCT over the cycle columns.
+  */
+  for (st_select_lex *sl=  spec->first_select(); sl; sl= sl->next_select())
+  {
+    spec->union_distinct= sl;
+    if (sl != spec->first_select())
+    {
+      sl->distinct= TRUE;
+      sl->with_all_modifier= FALSE;
+    }
+  }
+}
