@@ -296,15 +296,22 @@ int TABLE::delete_row()
 
   store_record(this, record[1]);
   vers_update_end();
-  int err= file->ha_update_row(record[1], record[0]);
-  /*
-     MDEV-23644: we get HA_ERR_FOREIGN_DUPLICATE_KEY iff we already got history
-     row with same trx_id which is the result of foreign key action, so we
-     don't need one more history row.
-  */
-  if (err == HA_ERR_FOREIGN_DUPLICATE_KEY)
-    return file->ha_delete_row(record[0]);
-  return err;
+  int err;
+  if ((err= file->extra(HA_EXTRA_REMEMBER_POS)))
+    return err;
+  if ((err= file->ha_update_row(record[1], record[0])))
+  {
+    /*
+      MDEV-23644: we get HA_ERR_FOREIGN_DUPLICATE_KEY iff we already got
+      history row with same trx_id which is the result of foreign key action,
+      so we don't need one more history row.
+    */
+    if (err == HA_ERR_FOREIGN_DUPLICATE_KEY)
+      file->ha_delete_row(record[0]);
+    else
+      return err;
+  }
+  return file->extra(HA_EXTRA_RESTORE_POS);
 }
 
 
@@ -352,6 +359,16 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
   bool delete_record= false;
   bool delete_while_scanning= table_list->delete_while_scanning;
   bool portion_of_time_through_update;
+  /*
+    TRUE if we are after the call to
+    select_lex->optimize_unflattened_subqueries(true) and before the
+    call to select_lex->optimize_unflattened_subqueries(false), to
+    ensure a call to
+    select_lex->optimize_unflattened_subqueries(false) happens which
+    avoid 2nd ps mem leaks when e.g. the first execution produces
+    empty result and the second execution produces a non-empty set
+  */
+  bool optimize_subqueries= FALSE;
 
   DBUG_ENTER("Sql_cmd_delete::delete_single_table");
 
@@ -388,9 +405,18 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
 
   thd->lex->promote_select_describe_flag_if_needed();
 
-  /* Apply the IN=>EXISTS transformation to all subqueries and optimize them. */
-  if (select_lex->optimize_unflattened_subqueries(false))
+  /*
+    Apply the IN=>EXISTS transformation to all constant subqueries
+    and optimize them.
+
+    It is too early to choose subquery optimization strategies without
+    an estimate of how many times the subquery will be executed so we
+    call optimize_unflattened_subqueries() with const_only= true, and
+    choose between materialization and in-to-exists later.
+  */
+  if (select_lex->optimize_unflattened_subqueries(true))
     DBUG_RETURN(TRUE);
+  optimize_subqueries= TRUE;
 
   const_cond= (!conds || conds->const_item());
   safe_update= (thd->variables.option_bits & OPTION_SAFE_UPDATES) &&
@@ -402,10 +428,10 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
     DBUG_RETURN(TRUE);
   }
 
-  const_cond_result= const_cond && (!conds || conds->val_int());
+  const_cond_result= const_cond && (!conds || conds->val_bool());
   if (unlikely(thd->is_error()))
   {
-    /* Error evaluating val_int(). */
+    /* Error evaluating val_bool(). */
     DBUG_RETURN(TRUE);
   }
 
@@ -441,6 +467,7 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
     if (thd->lex->describe)
       goto produce_explain_and_leave;
 
+    table->file->prepare_for_modify(false, false);
     if (likely(!(error=table->file->ha_delete_all_rows())))
     {
       /*
@@ -491,6 +518,9 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
 #ifdef WITH_PARTITION_STORAGE_ENGINE
   if (prune_partitions(thd, table, conds))
   {
+    if (optimize_subqueries && select_lex->optimize_unflattened_subqueries(false))
+      DBUG_RETURN(TRUE);
+    optimize_subqueries= FALSE;
     free_underlaid_joins(thd, select_lex);
 
     query_plan.set_no_partitions();
@@ -500,6 +530,13 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
     if (thd->binlog_for_noop_dml(transactional_table))
       DBUG_RETURN(1);
 
+    if (!thd->lex->current_select->leaf_tables_saved)
+    {
+      thd->lex->current_select->save_leaf_tables(thd);
+      thd->lex->current_select->leaf_tables_saved= true;
+      thd->lex->current_select->first_cond_optimization= 0;
+    }
+
     my_ok(thd, 0);
     DBUG_RETURN(0);
   }
@@ -523,6 +560,9 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
       goto produce_explain_and_leave;
 
     delete select;
+    if (select_lex->optimize_unflattened_subqueries(false))
+      DBUG_RETURN(TRUE);
+    optimize_subqueries= FALSE;
     free_underlaid_joins(thd, select_lex);
     /* 
       Error was already created by quick select evaluation (check_quick()).
@@ -540,6 +580,7 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
     {
       thd->lex->current_select->save_leaf_tables(thd);
       thd->lex->current_select->leaf_tables_saved= true;
+      thd->lex->current_select->first_cond_optimization= 0;
     }
 
     my_ok(thd, 0);
@@ -553,6 +594,9 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
     if (safe_update && !using_limit)
     {
       delete select;
+      if (optimize_subqueries && select_lex->optimize_unflattened_subqueries(false))
+        DBUG_RETURN(TRUE);
+      optimize_subqueries= FALSE;
       free_underlaid_joins(thd, select_lex);
       my_message(ER_UPDATE_WITHOUT_KEY_IN_SAFE_MODE,
                  ER_THD(thd, ER_UPDATE_WITHOUT_KEY_IN_SAFE_MODE), MYF(0));
@@ -562,7 +606,18 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
   if (options & OPTION_QUICK)
     (void) table->file->extra(HA_EXTRA_QUICK);
 
-  query_plan.scanned_rows= select? select->records: table->file->stats.records;
+  /*
+    Estimate the number of scanned rows and have it accessible in
+    JOIN::choose_subquery_plan() from the outer join through
+    JOIN::sql_cmd_dml
+  */
+  scanned_rows= query_plan.scanned_rows= select ?
+    select->records : table->file->stats.records;
+  select_lex->join->sql_cmd_dml= this;
+  DBUG_ASSERT(optimize_subqueries);
+  if (select_lex->optimize_unflattened_subqueries(false))
+    DBUG_RETURN(TRUE);
+  optimize_subqueries= FALSE;
   if (order)
   {
     table->update_const_key_parts(conds);
@@ -797,8 +852,8 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
           && !table->versioned()
           && table->file->has_transactions();
 
-  if (table->versioned(VERS_TIMESTAMP) || (table_list->has_period()))
-    table->file->prepare_for_insert(1);
+  table->file->prepare_for_modify(table->versioned(VERS_TIMESTAMP) ||
+                                  table_list->has_period(), true);
   DBUG_ASSERT(table->file->inited != handler::NONE);
 
   THD_STAGE_INFO(thd, stage_updating);
@@ -814,13 +869,17 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
                                               delete_history);
     if (delete_record)
     {
+      void *save_bulk_param= thd->bulk_param;
+      thd->bulk_param= nullptr;
       if (!delete_history && table->triggers &&
           table->triggers->process_triggers(thd, TRG_EVENT_DELETE,
                                             TRG_ACTION_BEFORE, FALSE))
       {
         error= 1;
+        thd->bulk_param= save_bulk_param;
         break;
       }
+      thd->bulk_param= save_bulk_param;
 
       // no LIMIT / OFFSET
       if (returning && result->send_data(returning->item_list) < 0)
@@ -851,13 +910,16 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
       if (likely(!error))
       {
         deleted++;
+	thd->bulk_param= nullptr;
         if (!delete_history && table->triggers &&
             table->triggers->process_triggers(thd, TRG_EVENT_DELETE,
                                               TRG_ACTION_AFTER, FALSE))
         {
           error= 1;
+          thd->bulk_param= save_bulk_param;
           break;
         }
+        thd->bulk_param= save_bulk_param;
         if (!--limit && using_limit)
         {
           error= -1;
@@ -918,6 +980,7 @@ cleanup:
   {
     thd->lex->current_select->save_leaf_tables(thd);
     thd->lex->current_select->leaf_tables_saved= true;
+    thd->lex->current_select->first_cond_optimization= false;
   }
 
   delete deltempfile;
@@ -977,6 +1040,8 @@ cleanup:
     DBUG_PRINT("info",("%ld records deleted",(long) deleted));
   }
   delete file_sort;
+  if (optimize_subqueries && select_lex->optimize_unflattened_subqueries(false))
+    DBUG_RETURN(TRUE);
   free_underlaid_joins(thd, select_lex);
   if (table->file->pushed_cond)
     table->file->cond_pop();
@@ -1000,6 +1065,9 @@ send_nothing_and_leave:
 
   delete select;
   delete file_sort;
+  if (!thd->is_error() && optimize_subqueries &&
+      select_lex->optimize_unflattened_subqueries(false))
+    DBUG_RETURN(TRUE);
   free_underlaid_joins(thd, select_lex);
   if (table->file->pushed_cond)
     table->file->cond_pop();
@@ -1030,7 +1098,7 @@ multi_delete::multi_delete(THD *thd_arg, TABLE_LIST *dt, uint num_of_tables_arg)
     num_of_tables(num_of_tables_arg), error(0),
     do_delete(0), transactional_tables(0), normal_tables(0), error_handled(0)
 {
-  tempfiles= (Unique **) thd_arg->calloc(sizeof(Unique *) * num_of_tables);
+  tempfiles= thd_arg->calloc<Unique*>(num_of_tables);
 }
 
 
@@ -1113,9 +1181,7 @@ multi_delete::initialize_tables(JOIN *join)
 	normal_tables= 1;
       tbl->prepare_triggers_for_delete_stmt_or_event();
       tbl->prepare_for_position();
-
-      if (tbl->versioned(VERS_TIMESTAMP))
-        tbl->file->prepare_for_insert(1);
+      tbl->file->prepare_for_modify(tbl->versioned(VERS_TIMESTAMP), true);
     }
     else if ((tab->type != JT_SYSTEM && tab->type != JT_CONST) &&
              walk == delete_tables)
