@@ -89,7 +89,11 @@ bool reload_acl_and_cache(THD *thd, unsigned long long options,
       allocate temporary THD for execution of acl_reload()/grant_reload().
     */
     if (unlikely(!thd) && (thd= (tmp_thd= new THD(0))))
+    {
       thd->store_globals();
+      thd->set_query_inner((char*) STRING_WITH_LEN("intern:reload_acl"),
+                           default_charset_info);
+    }
 
     if (likely(thd))
     {
@@ -220,7 +224,6 @@ bool reload_acl_and_cache(THD *thd, unsigned long long options,
     }
 #endif
   }
-#ifdef HAVE_QUERY_CACHE
   if (options & REFRESH_QUERY_CACHE_FREE)
   {
     query_cache.pack(thd);              // FLUSH QUERY CACHE
@@ -230,7 +233,6 @@ bool reload_acl_and_cache(THD *thd, unsigned long long options,
   {
     query_cache.flush();			// RESET QUERY CACHE
   }
-#endif /*HAVE_QUERY_CACHE*/
 
   DBUG_ASSERT(!thd || thd->locked_tables_mode ||
               !thd->mdl_context.has_locks() ||
@@ -375,7 +377,11 @@ bool reload_acl_and_cache(THD *thd, unsigned long long options,
   if (options & REFRESH_HOSTS)
     hostname_cache_refresh();
   if (thd && (options & REFRESH_STATUS))
-    refresh_status(thd);
+    refresh_status_legacy(thd);
+  if (thd && (options & REFRESH_SESSION_STATUS))
+    refresh_session_status(thd);
+  if ((options & REFRESH_GLOBAL_STATUS))
+    refresh_global_status();
   if (options & REFRESH_THREADS)
     thread_cache.flush();
 #ifdef HAVE_REPLICATION
@@ -671,6 +677,6 @@ static void disable_checkpoints(THD *thd)
   {
     thd->global_disable_checkpoint= 1;
     if (!global_disable_checkpoint++)
-      ha_checkpoint_state(1);                   // Disable checkpoints
+      ha_disable_internal_writes(1);                   // Disable checkpoints
   }
 }
