@@ -27,10 +27,6 @@
    dynamic functions, so this shouldn't be a real problem.
 */
 
-#ifdef USE_PRAGMA_IMPLEMENTATION
-#pragma implementation				// gcc: Class implementation
-#endif
-
 #include "mariadb.h"
 #include "sql_priv.h"
 #include "unireg.h"
@@ -98,7 +94,7 @@ static const char *init_syms(udf_func *tmp, char *nm)
     if (!opt_allow_suspicious_udfs)
       return nm;
     if (thd->variables.log_warnings)
-      sql_print_warning(ER_THD(thd, ER_CANT_FIND_DL_ENTRY), nm);
+      sql_print_warning(ER_DEFAULT(ER_CANT_FIND_DL_ENTRY), nm, tmp->name.str);
   }
   return 0;
 }
@@ -169,8 +165,9 @@ void udf_init()
   init_sql_alloc(key_memory_udf_mem, &mem, UDF_ALLOC_BLOCK_SIZE, 0, MYF(0));
   THD *new_thd = new THD(0);
   if (!new_thd ||
-      my_hash_init(key_memory_udf_mem,
-                   &udf_hash,system_charset_info,32,0,0,get_hash_key, NULL, 0))
+      my_hash_init(key_memory_udf_mem, &udf_hash,
+                   Lex_ident_routine::charset_info(),
+                   32,0,0,get_hash_key, NULL, 0))
   {
     sql_print_error("Can't allocate memory for udf structures");
     my_hash_free(&udf_hash);
@@ -179,7 +176,7 @@ void udf_init()
     DBUG_VOID_RETURN;
   }
   initialized = 1;
-  new_thd->thread_stack= (char*) &new_thd;
+  new_thd->thread_stack= (void*) &new_thd;      // Big stack
   new_thd->store_globals();
   new_thd->set_query_inner((char*) STRING_WITH_LEN("intern:udf_init"),
                            default_charset_info);
@@ -208,10 +205,11 @@ void udf_init()
   while (!(error= read_record_info.read_record()))
   {
     DBUG_PRINT("info",("init udf record"));
-    LEX_CSTRING name;
-    name.str=get_field(&mem, table->field[0]);
-    name.length = (uint) safe_strlen(name.str);
-    char *dl_name= get_field(&mem, table->field[2]);
+    DBUG_ASSERT(!(new_thd->variables.sql_mode & MODE_PAD_CHAR_TO_FULL_LENGTH));
+    DBUG_ASSERT(table->field[0]->get_thd() == new_thd);
+    DBUG_ASSERT(table->field[2]->get_thd() == new_thd);
+    LEX_CSTRING name= table->field[0]->val_lex_string_strmake(&mem);
+    LEX_CSTRING dl_name= table->field[2]->val_lex_string_strmake(&mem);
     bool new_dl=0;
     Item_udftype udftype=UDFTYPE_FUNCTION;
     if (table->s->fields >= 4)			// New func table
@@ -224,7 +222,8 @@ void udf_init()
 
       On windows we must check both FN_LIBCHAR and '/'.
     */
-    if (!name.str || !dl_name || check_valid_path(dl_name, strlen(dl_name)) ||
+    if (!name.length || !dl_name.length ||
+        check_valid_path(dl_name.str, dl_name.length) ||
         check_string_char_length(&name, 0, NAME_CHAR_LEN,
                                  system_charset_info, 1))
     {
@@ -234,7 +233,7 @@ void udf_init()
     }
 
     if (!(tmp= add_udf(&name,(Item_result) table->field[1]->val_int(),
-                       dl_name, udftype)))
+                       dl_name.str, udftype)))
     {
       sql_print_error("Can't alloc memory for udf function: '%.64s'", name.str);
       continue;
@@ -249,7 +248,7 @@ void udf_init()
       if (!(dl= dlopen(dlpath, RTLD_NOW)))
       {
 	/* Print warning to log */
-        sql_print_error(ER_THD(new_thd, ER_CANT_OPEN_LIBRARY),
+        sql_print_error(ER_DEFAULT(ER_CANT_OPEN_LIBRARY),
                         tmp->dl, errno, my_dlerror(dlpath));
 	/* Keep the udf in the hash so that we can remove it later */
 	continue;
@@ -262,7 +261,8 @@ void udf_init()
       const char *missing;
       if ((missing= init_syms(tmp, buf)))
       {
-        sql_print_error(ER_THD(new_thd, ER_CANT_FIND_DL_ENTRY), missing);
+        sql_print_error(ER_DEFAULT(ER_CANT_FIND_DL_ENTRY), missing,
+                        tmp->name.str);
         del_udf(tmp);
         if (new_dl)
           dlclose(dl);
@@ -602,7 +602,7 @@ int mysql_create_function(THD *thd,udf_func *udf)
     const char *missing;
     if ((missing= init_syms(udf, buf)))
     {
-      my_error(ER_CANT_FIND_DL_ENTRY, MYF(0), missing);
+      my_error(ER_CANT_FIND_DL_ENTRY, MYF(0), missing, udf->dl);
       goto err;
     }
   }
