@@ -29,37 +29,38 @@ Window_spec::check_window_names(List_iterator_fast<Window_spec> &it)
 {
   if (window_names_are_checked)
     return false;
-  const char *name= this->name();
-  const char *ref_name= window_reference();
+  const Lex_ident_window name= this->name();
+  const Lex_ident_window ref_name= window_reference();
   it.rewind();
   Window_spec *win_spec;
   while((win_spec= it++) && win_spec != this)
   {
-    const char *win_spec_name= win_spec->name();
-    if (!win_spec_name)
+    const Lex_ident_window win_spec_name= win_spec->name();
+    if (!win_spec_name.str)
       break;
-    if (name && my_strcasecmp(system_charset_info, name, win_spec_name) == 0)
+    if (name.str && name.streq(win_spec_name))
     {
-      my_error(ER_DUP_WINDOW_NAME, MYF(0), name);
+      my_error(ER_DUP_WINDOW_NAME, MYF(0), name.str);
       return true;
     }
-    if (ref_name &&
-        my_strcasecmp(system_charset_info, ref_name, win_spec_name) == 0)
+    if (ref_name.str && win_spec_name.streq(ref_name))
     {
       if (partition_list->elements)
       {
         my_error(ER_PARTITION_LIST_IN_REFERENCING_WINDOW_SPEC, MYF(0),
-                 ref_name);
+                 ref_name.str);
         return true;
       }
       if (win_spec->order_list->elements && order_list->elements)
       {
-        my_error(ER_ORDER_LIST_IN_REFERENCING_WINDOW_SPEC, MYF(0), ref_name);
+        my_error(ER_ORDER_LIST_IN_REFERENCING_WINDOW_SPEC, MYF(0),
+                 ref_name.str);
         return true;
       } 
       if (win_spec->window_frame)
       {
-        my_error(ER_WINDOW_FRAME_IN_REFERENCED_WINDOW_SPEC, MYF(0), ref_name);
+        my_error(ER_WINDOW_FRAME_IN_REFERENCED_WINDOW_SPEC, MYF(0),
+                 ref_name.str);
         return true;
       }
       referenced_win_spec= win_spec;
@@ -69,9 +70,9 @@ Window_spec::check_window_names(List_iterator_fast<Window_spec> &it)
         order_list= win_spec->order_list;
     }
   }
-  if (ref_name && !referenced_win_spec)
+  if (ref_name.str && !referenced_win_spec)
   {
-    my_error(ER_WRONG_WINDOW_SPEC_NAME, MYF(0), ref_name);
+    my_error(ER_WRONG_WINDOW_SPEC_NAME, MYF(0), ref_name.str);
     return true;
   }
   window_names_are_checked= true;
@@ -221,7 +222,7 @@ setup_windows(THD *thd, Ref_ptr_array ref_pointer_array, TABLE_LIST *tables,
     uint elems= win_specs.elements;
     while ((win_spec= it++) && i++ < elems)
     {
-      if (win_spec->name() == NULL)
+      if (win_spec->name().str == NULL)
       {
         it.remove();
         win_specs.push_back(win_spec);
@@ -432,6 +433,16 @@ ORDER *st_select_lex::find_common_window_func_partition_fields(THD *thd)
 #define CMP_GT_C       1    // Greater than and compatible
 #define CMP_GT         2    // Greater then
 
+
+/*
+  This function is used for sorting ORDER/PARTITION BY clauses of window
+  functions and so must implement an order relation on ORDER BY clauses"
+
+  It is called by a sorting function.
+  The function return's CMP_EQ (=0) if the values are identical.
+  If not equal, it returns a stable value < or > than 0.
+*/
+
 static
 int compare_order_elements(ORDER *ord1, int weight1,
                            ORDER *ord2, int weight2)
@@ -640,7 +651,7 @@ int compare_window_funcs_by_window_specs(Item_window_func *win_func1,
       Partition lists contain the same elements. 
       Let's use only one of the lists.
     */
-    if (!win_spec1->name() && win_spec2->name())
+    if (!win_spec1->name().str && win_spec2->name().str)
     {
       win_spec1->save_partition_list= win_spec1->partition_list;
       win_spec1->partition_list= win_spec2->partition_list;
@@ -663,7 +674,7 @@ int compare_window_funcs_by_window_specs(Item_window_func *win_func1,
        Order lists contain the same elements.
        Let's use only one of the lists.
     */
-    if (!win_spec1->name() && win_spec2->name())
+    if (!win_spec1->name().str && win_spec2->name().str)
     {
       win_spec1->save_order_list= win_spec2->order_list;
       win_spec1->order_list= win_spec2->order_list;
@@ -681,7 +692,7 @@ int compare_window_funcs_by_window_specs(Item_window_func *win_func1,
       return cmp;
 
     /* Window frames are equal. Let's use only one of them. */
-    if (!win_spec1->name() && win_spec2->name())
+    if (!win_spec1->name().str && win_spec2->name().str)
       win_spec1->window_frame= win_spec2->window_frame;
     else
       win_spec2->window_frame= win_spec1->window_frame;
@@ -2851,6 +2862,7 @@ bool compute_window_func(THD *thd,
 {
   List_iterator_fast<Item_window_func> iter_win_funcs(window_functions);
   List_iterator_fast<Cursor_manager> iter_cursor_managers(cursor_managers);
+  bool ret= false;
   uint err;
 
   READ_RECORD info;
@@ -2913,18 +2925,27 @@ bool compute_window_func(THD *thd,
       /* Check if we found any error in the window function while adding values
          through cursors. */
       if (unlikely(thd->is_error() || thd->is_killed()))
+      {
+        ret= true;
         break;
-
+      }
 
       /* Return to current row after notifying cursors for each window
          function. */
-      tbl->file->ha_rnd_pos(tbl->record[0], rowid_buf);
+      if (tbl->file->ha_rnd_pos(tbl->record[0], rowid_buf))
+      {
+        ret= true;
+        break;
+      }
     }
 
     /* We now have computed values for each window function. They can now
        be saved in the current row. */
-    save_window_function_values(window_functions, tbl, rowid_buf);
-
+    if (save_window_function_values(window_functions, tbl, rowid_buf))
+    {
+      ret= true;
+      break;
+    }
     rownum++;
   }
 
@@ -2932,7 +2953,7 @@ bool compute_window_func(THD *thd,
   partition_trackers.delete_elements();
   end_read_record(&info);
 
-  return false;
+  return ret;
 }
 
 /* Make a list that is a concation of two lists of ORDER elements */
@@ -3128,8 +3149,10 @@ bool Window_funcs_sort::setup(THD *thd, SQL_SELECT *sel,
      */
     ORDER *order= (ORDER *)alloc_root(thd->mem_root, sizeof(ORDER));
     memset(order, 0, sizeof(*order));
-    Item *item= new (thd->mem_root) Item_temptable_field(thd,
-                                                    join_tab->table->field[0]);
+    Item_field *item=
+        new (thd->mem_root) Item_field(thd, join_tab->table->field[0]);
+    if (item)
+      item->set_refers_to_temp_table();
     order->item= (Item **)alloc_root(thd->mem_root, 2 * sizeof(Item *));
     order->item[1]= NULL;
     order->item[0]= item;
