@@ -25,10 +25,6 @@
   it creates unsolved link dependencies on some platforms.
 */
 
-#ifdef USE_PRAGMA_IMPLEMENTATION
-#pragma implementation				// gcc: Class implementation
-#endif
-
 #define VER "1.1"
 #include "mariadb.h"
 #if !defined(TZINFO2SQL) && !defined(TESTTIME)
@@ -52,28 +48,19 @@
 #include <mysql/psi/mysql_file.h>
 #include "lock.h"                               // MYSQL_LOCK_IGNORE_FLUSH,
                                                 // MYSQL_LOCK_IGNORE_TIMEOUT
-
 /*
-  Now we don't use abbreviations in server but we will do this in future.
-*/
-#if defined(TZINFO2SQL) || defined(TESTTIME)
-#define ABBR_ARE_USED
-#else
-#if !defined(DBUG_OFF)
-/* Let use abbreviations for debug purposes */
-#undef ABBR_ARE_USED
-#define ABBR_ARE_USED
-#endif /* !defined(DBUG_OFF) */
-#endif /* defined(TZINFO2SQL) || defined(TESTTIME) */
+  Internal server time. Needed to interface with my_time_t, which can
+  be either long, ulong or long long depending on hardware or os.
+ */
+
+typedef longlong my_int_time_t;
 
 /* Structure describing local time type (e.g. Moscow summer time (MSD)) */
 typedef struct ttinfo
 {
   long tt_gmtoff; // Offset from UTC in seconds
   uint tt_isdst;   // Is daylight saving time or not. Used to set tm_isdst
-#ifdef ABBR_ARE_USED
   uint tt_abbrind; // Index of start of abbreviation for this time type.
-#endif
   /*
     We don't use tt_ttisstd and tt_ttisgmt members of original elsie-code
     struct since we don't support POSIX-style TZ descriptions in variables.
@@ -83,7 +70,7 @@ typedef struct ttinfo
 /* Structure describing leap-second corrections. */
 typedef struct lsinfo
 {
-  my_time_t ls_trans; // Transition time
+  my_int_time_t ls_trans; // Transition time
   long      ls_corr;  // Correction to apply
 } LS_INFO;
 
@@ -117,13 +104,11 @@ typedef struct st_time_zone_info
   uint charcnt;  // Number of characters used for abbreviations
   uint revcnt;   // Number of transition descr. for TIME->my_time_t conversion
   /* The following are dynamical arrays are allocated in MEM_ROOT */
-  my_time_t *ats;       // Times of transitions between time types
+  my_int_time_t *ats;       // Times of transitions between time types
   uchar	*types; // Local time types for transitions
   TRAN_TYPE_INFO *ttis; // Local time types descriptions
-#ifdef ABBR_ARE_USED
   /* Storage for local time types abbreviations. They are stored as ASCIIZ */
   char *chars;
-#endif
   /*
     Leap seconds corrections descriptions, this array is shared by
     all time zones who use leap seconds.
@@ -134,7 +119,7 @@ typedef struct st_time_zone_info
     ranges on which shifted my_time_t -> my_time_t mapping is linear or undefined.
     Used for tm -> my_time_t conversion.
   */
-  my_time_t *revts;
+  my_int_time_t *revts;
   REVT_INFO *revtis;
   /*
     Time type which is used for times smaller than first transition or if
@@ -177,11 +162,9 @@ tz_load(const char *name, TIME_ZONE_INFO *sp, MEM_ROOT *storage)
     union
     {
       struct tzhead tzhead;
-      uchar buf[sizeof(struct tzhead) + sizeof(my_time_t) * TZ_MAX_TIMES +
+      uchar buf[sizeof(struct tzhead) + sizeof(my_int_time_t) * TZ_MAX_TIMES +
                 TZ_MAX_TIMES + sizeof(TRAN_TYPE_INFO) * TZ_MAX_TYPES +
-#ifdef ABBR_ARE_USED
                MY_MAX(TZ_MAX_CHARS + 1, (2 * (MY_TZNAME_MAX + 1))) +
-#endif
                sizeof(LS_INFO) * TZ_MAX_LEAPS];
     } u;
     uint ttisstdcnt;
@@ -222,26 +205,22 @@ tz_load(const char *name, TIME_ZONE_INFO *sp, MEM_ROOT *storage)
 
     if (!(tzinfo_buf= (char *)alloc_root(storage,
                                          ALIGN_SIZE(sp->timecnt *
-                                                    sizeof(my_time_t)) +
+                                                    sizeof(*sp->ats)) +
                                          ALIGN_SIZE(sp->timecnt) +
                                          ALIGN_SIZE(sp->typecnt *
                                                     sizeof(TRAN_TYPE_INFO)) +
-#ifdef ABBR_ARE_USED
                                          ALIGN_SIZE(sp->charcnt+1) +
-#endif
                                          sp->leapcnt * sizeof(LS_INFO))))
       return 1;
 
-    sp->ats= (my_time_t *)tzinfo_buf;
-    tzinfo_buf+= ALIGN_SIZE(sp->timecnt * sizeof(my_time_t));
+    sp->ats= (my_int_time_t*) tzinfo_buf;
+    tzinfo_buf+= ALIGN_SIZE(sp->timecnt * sizeof(sp->ats[0]));
     sp->types= (uchar *)tzinfo_buf;
     tzinfo_buf+= ALIGN_SIZE(sp->timecnt);
     sp->ttis= (TRAN_TYPE_INFO *)tzinfo_buf;
     tzinfo_buf+= ALIGN_SIZE(sp->typecnt * sizeof(TRAN_TYPE_INFO));
-#ifdef ABBR_ARE_USED
     sp->chars= tzinfo_buf;
     tzinfo_buf+= ALIGN_SIZE(sp->charcnt+1);
-#endif
     sp->lsis= (LS_INFO *)tzinfo_buf;
 
     for (i= 0; i < sp->timecnt; i++, p+= 4)
@@ -326,9 +305,9 @@ tz_load(const char *name, TIME_ZONE_INFO *sp, MEM_ROOT *storage)
 static my_bool
 prepare_tz_info(TIME_ZONE_INFO *sp, MEM_ROOT *storage)
 {
-  my_time_t cur_t= MY_TIME_T_MIN;
-  my_time_t cur_l, end_t, UNINIT_VAR(end_l);
-  my_time_t cur_max_seen_l= MY_TIME_T_MIN;
+  my_int_time_t cur_t= MY_TIME_T_MIN;
+  my_int_time_t cur_l, end_t, UNINIT_VAR(end_l);
+  my_int_time_t cur_max_seen_l= MY_TIME_T_MIN;
   long cur_offset, cur_corr, cur_off_and_corr;
   uint next_trans_idx, next_leap_idx;
   uint i;
@@ -337,7 +316,7 @@ prepare_tz_info(TIME_ZONE_INFO *sp, MEM_ROOT *storage)
     we don't know table sizes ahead. (Well we can estimate their
     upper bound but this will take extra space.)
   */
-  my_time_t revts[TZ_MAX_REV_RANGES];
+  my_int_time_t revts[TZ_MAX_REV_RANGES];
   REVT_INFO revtis[TZ_MAX_REV_RANGES];
 
   /*
@@ -496,14 +475,15 @@ prepare_tz_info(TIME_ZONE_INFO *sp, MEM_ROOT *storage)
   revts[sp->revcnt]= end_l;
 
   /* Allocate arrays of proper size in sp and copy result there */
-  if (!(sp->revts= (my_time_t *)alloc_root(storage,
-                                  sizeof(my_time_t) * (sp->revcnt + 1))) ||
+  if (!(sp->revts= (my_int_time_t*) alloc_root(storage,
+                                               sizeof(*sp->revts) *
+                                               (sp->revcnt + 1))) ||
       !(sp->revtis= (REVT_INFO *)alloc_root(storage,
-                                  sizeof(REVT_INFO) * sp->revcnt)))
+                                            sizeof(REVT_INFO) * sp->revcnt)))
     return 1;
 
-  memcpy(sp->revts, revts, sizeof(my_time_t) * (sp->revcnt + 1));
-  memcpy(sp->revtis, revtis, sizeof(REVT_INFO) * sp->revcnt);
+  memcpy(sp->revts, revts, sizeof(sp->revts[0]) * (sp->revcnt + 1));
+  memcpy(sp->revtis, revtis, sizeof(sp->revtis[0]) * sp->revcnt);
 
   return 0;
 }
@@ -633,7 +613,7 @@ sec_to_TIME(MYSQL_TIME * tmp, my_time_t t, long offset)
     Index of range to which t belongs
 */
 static uint
-find_time_range(my_time_t t, const my_time_t *range_boundaries,
+find_time_range(my_int_time_t t, const my_int_time_t *range_boundaries,
                 uint higher_bound)
 {
   uint i, lower_bound= 0;
@@ -788,18 +768,16 @@ gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t sec_in_utc, const TIME_ZONE_INFO *sp)
   RETURN VALUE
     Seconds since epoch time representation.
 */
-static my_time_t
+static longlong
 sec_since_epoch(int year, int mon, int mday, int hour, int min ,int sec)
 {
-  /* Guard against my_time_t overflow(on system with 32 bit my_time_t) */
-  DBUG_ASSERT(!(year == TIMESTAMP_MAX_YEAR && mon == 1 && mday > 17));
 #ifndef WE_WANT_TO_HANDLE_UNORMALIZED_DATES
   /*
     It turns out that only whenever month is normalized or unnormalized
     plays role.
   */
   DBUG_ASSERT(mon > 0 && mon < 13);
-  long days= year * DAYS_PER_NYEAR - EPOCH_YEAR * DAYS_PER_NYEAR +
+  longlong days= year * DAYS_PER_NYEAR - EPOCH_YEAR * DAYS_PER_NYEAR +
              LEAPS_THRU_END_OF(year - 1) -
              LEAPS_THRU_END_OF(EPOCH_YEAR - 1);
   days+= mon_starts[isleap(year)][mon - 1];
@@ -814,8 +792,8 @@ sec_since_epoch(int year, int mon, int mday, int hour, int min ,int sec)
 #endif
   days+= mday - 1;
 
-  return ((days * HOURS_PER_DAY + hour) * MINS_PER_HOUR + min) *
-         SECS_PER_MIN + sec;
+  return ((longlong((days * HOURS_PER_DAY + hour) * MINS_PER_HOUR + min)) *
+          SECS_PER_MIN + sec);
 }
 
 /*
@@ -893,10 +871,14 @@ sec_since_epoch(int year, int mon, int mday, int hour, int min ,int sec)
     0 in case of error.
 */
 
+#if TIMESTAMP_MAX_DAY <= 2
+#error TIMESTAMP_MAX_DAY has to be > 2 for TIME_to_gmt_sec to work
+#endif
+
 static my_time_t
 TIME_to_gmt_sec(const MYSQL_TIME *t, const TIME_ZONE_INFO *sp, uint *error_code)
 {
-  my_time_t local_t;
+  my_int_time_t local_t;
   uint saved_seconds;
   uint i;
   int shift= 0;
@@ -918,7 +900,7 @@ TIME_to_gmt_sec(const MYSQL_TIME *t, const TIME_ZONE_INFO *sp, uint *error_code)
 
   /*
     NOTE: to convert full my_time_t range we do a shift of the
-    boundary dates here to avoid overflow of my_time_t.
+    boundary dates here to avoid overflow of my_time_t range.
     We use alike approach in my_system_gmt_sec().
 
     However in that function we also have to take into account
@@ -926,19 +908,19 @@ TIME_to_gmt_sec(const MYSQL_TIME *t, const TIME_ZONE_INFO *sp, uint *error_code)
     uses localtime_r(), which doesn't work with negative values correctly
     on platforms with unsigned time_t (QNX). Here we don't use localtime()
     => we negative values of local_t are ok.
-  */
+a  */
 
-  if ((t->year == TIMESTAMP_MAX_YEAR) && (t->month == 1) && t->day > 4)
+  if (t->year == TIMESTAMP_MAX_YEAR && t->month == TIMESTAMP_MAX_MONTH &&
+      t->day > 2)
   {
     /*
       We will pass (t->day - shift) to sec_since_epoch(), and
       want this value to be a positive number, so we shift
-      only dates > 4.01.2038 (to avoid owerflow).
+      only dates > max_day (to avoid owerflow).
     */
     shift= 2;
   }
 
-
   local_t= sec_since_epoch(t->year, t->month, (t->day - shift),
                            t->hour, t->minute,
                            saved_seconds ? 0 : t->second);
@@ -966,8 +948,8 @@ TIME_to_gmt_sec(const MYSQL_TIME *t, const TIME_ZONE_INFO *sp, uint *error_code)
   */
   if (shift)
   {
-    if (local_t > (my_time_t) (TIMESTAMP_MAX_VALUE - shift * SECS_PER_DAY +
-                               sp->revtis[i].rt_offset - saved_seconds))
+    if (local_t > (longlong) (TIMESTAMP_MAX_VALUE - shift * SECS_PER_DAY +
+                              sp->revtis[i].rt_offset - saved_seconds))
     {
       *error_code= ER_WARN_DATA_OUT_OF_RANGE;
       DBUG_RETURN(0);                           /* my_time_t overflow */
@@ -989,14 +971,13 @@ TIME_to_gmt_sec(const MYSQL_TIME *t, const TIME_ZONE_INFO *sp, uint *error_code)
   else
     local_t= local_t - sp->revtis[i].rt_offset + saved_seconds;
 
-  /* check for TIMESTAMP_MAX_VALUE was already done above */
-  if (local_t < TIMESTAMP_MIN_VALUE)
+  if (local_t < TIMESTAMP_MIN_VALUE || local_t > TIMESTAMP_MAX_VALUE)
   {
     local_t= 0;
     *error_code= ER_WARN_DATA_OUT_OF_RANGE;
   }
 
-  DBUG_RETURN(local_t);
+  DBUG_RETURN((my_time_t) local_t);
 }
 
 
@@ -1028,9 +1009,12 @@ class Time_zone_system : public Time_zone
 {
 public:
   Time_zone_system() = default;                       /* Remove gcc warning */
-  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, uint *error_code) const override;
+  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
+                            uint *error_code) const override;
   void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
   const String * get_name() const override;
+  void get_timezone_information(struct my_tz* curr_tz,
+                                const MYSQL_TIME *local_TIME) const override;
 };
 
 
@@ -1089,7 +1073,7 @@ void
 Time_zone_system::gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const
 {
   struct tm tmp_tm;
-  time_t tmp_t= (time_t)t;
+  time_t tmp_t= (time_t) t;
 
   localtime_r(&tmp_t, &tmp_tm);
   localtime_to_TIME(tmp, &tmp_tm);
@@ -1098,6 +1082,15 @@ Time_zone_system::gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const
 }
 
 
+void
+Time_zone_system::get_timezone_information(struct my_tz* curr_tz, const MYSQL_TIME *local_TIME) const
+{
+  uint error;
+  time_t time_sec= TIME_to_gmt_sec(local_TIME, &error);
+  my_tzinfo(time_sec, curr_tz);
+}
+
+
 /*
   Get name of time zone
 
@@ -1125,9 +1118,11 @@ class Time_zone_utc : public Time_zone
 public:
   Time_zone_utc() = default;                          /* Remove gcc warning */
   my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
-                                    uint *error_code) const override;
+                            uint *error_code) const override;
   void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
   const String * get_name() const override;
+  void get_timezone_information(struct my_tz* curr_tz,
+                                const MYSQL_TIME *local_TIME) const override;
 };
 
 
@@ -1175,6 +1170,13 @@ Time_zone_utc::gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const
   adjust_leap_second(tmp);
 }
 
+void
+Time_zone_utc::get_timezone_information(struct my_tz* curr_tz, const MYSQL_TIME *local_TIME) const
+{
+  strmake_buf(curr_tz->abbreviation, "UTC");
+  curr_tz->seconds_offset= 0;
+}
+
 
 /*
   Get name of time zone
@@ -1207,9 +1209,12 @@ class Time_zone_db : public Time_zone
 {
 public:
   Time_zone_db(TIME_ZONE_INFO *tz_info_arg, const String * tz_name_arg);
-  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, uint *error_code) const override;
+  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
+                            uint *error_code) const override;
   void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
   const String * get_name() const override;
+  void get_timezone_information(struct my_tz* curr_tz,
+                                const MYSQL_TIME *local_TIME) const override;
 private:
   TIME_ZONE_INFO *tz_info;
   const String *tz_name;
@@ -1280,6 +1285,22 @@ Time_zone_db::gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const
   adjust_leap_second(tmp);
 }
 
+void
+Time_zone_db::get_timezone_information(struct my_tz* curr_tz, const MYSQL_TIME *local_TIME) const
+{
+  uint error;
+  my_time_t sec_in_utc;
+  const TRAN_TYPE_INFO *ttisp;
+
+  /* Get seconds since epoch. */
+  sec_in_utc= TIME_to_gmt_sec(local_TIME, &error);
+  /* Get local timezone information. */
+  ttisp= find_transition_type(sec_in_utc, tz_info);
+
+  curr_tz->seconds_offset= ttisp->tt_gmtoff;
+  strmake_buf(curr_tz->abbreviation, &(tz_info->chars[ttisp->tt_abbrind]));
+}
+
 
 /*
   Get name of time zone
@@ -1306,9 +1327,11 @@ class Time_zone_offset : public Time_zone
 public:
   Time_zone_offset(long tz_offset_arg);
   my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
-                                    uint *error_code) const override;
-  void   gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
+                            uint *error_code) const override;
+  void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
   const String * get_name() const override;
+  void get_timezone_information(struct my_tz* curr_tz,
+                                const MYSQL_TIME *local_TIME) const override;
   /*
     This have to be public because we want to be able to access it from
     my_offset_tzs_get_key() function
@@ -1361,7 +1384,7 @@ Time_zone_offset::Time_zone_offset(long tz_offset_arg):
 my_time_t
 Time_zone_offset::TIME_to_gmt_sec(const MYSQL_TIME *t, uint *error_code) const
 {
-  my_time_t local_t;
+  my_int_time_t local_t;
   int shift= 0;
 
   /*
@@ -1394,7 +1417,7 @@ Time_zone_offset::TIME_to_gmt_sec(const MYSQL_TIME *t, uint *error_code) const
   }
 
   if (local_t >= TIMESTAMP_MIN_VALUE && local_t <= TIMESTAMP_MAX_VALUE)
-    return local_t;
+    return (my_time_t) local_t;
 
   /* range error*/
   *error_code= ER_WARN_DATA_OUT_OF_RANGE;
@@ -1434,6 +1457,14 @@ Time_zone_offset::get_name() const
   return &name;
 }
 
+void
+Time_zone_offset::get_timezone_information(struct my_tz* curr_tz, const MYSQL_TIME *local_TIME) const
+{
+  curr_tz->seconds_offset= offset;
+  const char *name= get_name()->ptr();
+  strmake_buf(curr_tz->abbreviation, name);
+}
+
 
 static Time_zone_utc tz_UTC;
 static Time_zone_system tz_SYSTEM;
@@ -1606,7 +1637,7 @@ my_tz_init(THD *org_thd, const char *default_tzname, my_bool bootstrap)
   THD *thd;
   TABLE_LIST tz_tables[1+MY_TZ_TABLES_COUNT];
   TABLE *table;
-  const LEX_CSTRING tmp_table_name= { STRING_WITH_LEN("time_zone_leap_second") };
+  const Lex_ident_table tmp_table_name= "time_zone_leap_second"_Lex_ident_table;
   Tz_names_entry *tmp_tzname;
   my_bool return_val= 1;
   int res;
@@ -1858,7 +1889,7 @@ tz_load_from_open_tables(const String *tz_name, TABLE_LIST *tz_tables)
   Time_zone *return_val= 0;
   int res;
   uint tzid, ttid;
-  my_time_t ttime;
+  my_int_time_t ttime;
   char buff[MAX_FIELD_WIDTH];
   uchar keybuff[32];
   Field *field;
@@ -1869,18 +1900,17 @@ tz_load_from_open_tables(const String *tz_name, TABLE_LIST *tz_tables)
     Temporary arrays that are used for loading of data for filling
     TIME_ZONE_INFO structure
   */
-  my_time_t ats[TZ_MAX_TIMES];
+  my_int_time_t ats[TZ_MAX_TIMES];
   uchar types[TZ_MAX_TIMES];
   TRAN_TYPE_INFO ttis[TZ_MAX_TYPES];
-#ifdef ABBR_ARE_USED
   char chars[MY_MAX(TZ_MAX_CHARS + 1, (2 * (MY_TZNAME_MAX + 1)))];
-#endif
   /*
     Used as a temporary tz_info until we decide that we actually want to
     allocate and keep the tz info and tz name in tz_storage.
   */
   TIME_ZONE_INFO tmp_tz_info;
   memset(&tmp_tz_info, 0, sizeof(TIME_ZONE_INFO));
+  memset(ttis, 0, sizeof(ttis));
 
   DBUG_ENTER("tz_load_from_open_tables");
 
@@ -1979,7 +2009,6 @@ tz_load_from_open_tables(const String *tz_name, TABLE_LIST *tz_tables)
     ttis[ttid].tt_gmtoff= (long)table->field[2]->val_int();
     ttis[ttid].tt_isdst= (table->field[3]->val_int() > 0);
 
-#ifdef ABBR_ARE_USED
     // FIXME should we do something with duplicates here ?
     table->field[4]->val_str(&abbr, &abbr);
     if (tmp_tz_info.charcnt + abbr.length() + 1 > sizeof(chars))
@@ -1999,11 +2028,6 @@ tz_load_from_open_tables(const String *tz_name, TABLE_LIST *tz_tables)
       ("time_zone_transition_type table: tz_id=%u tt_id=%u tt_gmtoff=%ld "
        "abbr='%s' tt_isdst=%u", tzid, ttid, ttis[ttid].tt_gmtoff,
        chars + ttis[ttid].tt_abbrind, ttis[ttid].tt_isdst));
-#else
-    DBUG_PRINT("info",
-      ("time_zone_transition_type table: tz_id=%u tt_id=%u tt_gmtoff=%ld "
-       "tt_isdst=%u", tzid, ttid, ttis[ttid].tt_gmtoff, ttis[ttid].tt_isdst));
-#endif
 
     /* ttid is increasing because we are reading using index */
     if (ttid < tmp_tz_info.typecnt)
@@ -2041,7 +2065,7 @@ tz_load_from_open_tables(const String *tz_name, TABLE_LIST *tz_tables)
                                       (key_part_map)1, HA_READ_KEY_EXACT);
   while (!res)
   {
-    ttime= (my_time_t)table->field[1]->val_int();
+    ttime= (my_int_time_t) table->field[1]->val_int();
     ttid= (uint)table->field[2]->val_int();
 
     if (tmp_tz_info.timecnt + 1 > TZ_MAX_TIMES)
@@ -2116,12 +2140,10 @@ tz_load_from_open_tables(const String *tz_name, TABLE_LIST *tz_tables)
     Now we will allocate memory and init TIME_ZONE_INFO structure.
   */
   if (!(alloc_buff= (char*) alloc_root(&tz_storage,
-                                       ALIGN_SIZE(sizeof(my_time_t) *
+                                       ALIGN_SIZE(sizeof(my_int_time_t) *
                                                   tz_info->timecnt) +
                                        ALIGN_SIZE(tz_info->timecnt) +
-#ifdef ABBR_ARE_USED
                                        ALIGN_SIZE(tz_info->charcnt) +
-#endif
                                        sizeof(TRAN_TYPE_INFO) *
                                        tz_info->typecnt)))
   {
@@ -2129,17 +2151,15 @@ tz_load_from_open_tables(const String *tz_name, TABLE_LIST *tz_tables)
     goto end;
   }
 
-  tz_info->ats= (my_time_t *) alloc_buff;
-  memcpy(tz_info->ats, ats, tz_info->timecnt * sizeof(my_time_t));
-  alloc_buff+= ALIGN_SIZE(sizeof(my_time_t) * tz_info->timecnt);
+  tz_info->ats= (my_int_time_t *) alloc_buff;
+  memcpy(tz_info->ats, ats, tz_info->timecnt * sizeof(tz_info->ats[0]));
+  alloc_buff+= ALIGN_SIZE(sizeof(tz_info->ats[0]) * tz_info->timecnt);
   tz_info->types= (uchar *)alloc_buff;
   memcpy(tz_info->types, types, tz_info->timecnt);
   alloc_buff+= ALIGN_SIZE(tz_info->timecnt);
-#ifdef ABBR_ARE_USED
   tz_info->chars= alloc_buff;
   memcpy(tz_info->chars, chars, tz_info->charcnt);
   alloc_buff+= ALIGN_SIZE(tz_info->charcnt);
-#endif
   tz_info->ttis= (TRAN_TYPE_INFO *)alloc_buff;
   memcpy(tz_info->ttis, ttis, tz_info->typecnt * sizeof(TRAN_TYPE_INFO));
 
@@ -2382,6 +2402,59 @@ void Time_zone::adjust_leap_second(MYSQL_TIME *t)
     t->second= 59;
 }
 
+
+/*
+  @brief
+    Check if timestamp<->datetime conversion is guaranteed to be reversible
+    around the given time value.
+*/
+
+bool Time_zone::is_monotone_continuous_around(my_time_t sec) const
+{
+  const my_time_t width= 24 * 60 * 60;
+  /*
+    Let's check that the TIMESTAMP range [sec - width, sec + width]
+    converts back to a DATETIME range [dtmin, dtmax], and the difference
+    (dtmax-dtmin), calculated using DATETIME arithmetics,
+    is also equal to exactly 2*width.
+
+    This should almost guarantee that the TIMESTAMP range around sec
+    is monotone and continuous and thus has no DST changes or leap seconds.
+
+    It's still possible that there are two or more timezone offset changes
+    in the given range. But there are no such time zones according to
+    our knowledge.
+    Note, DST changes and leap seconds do not interfere on a short time_t range:
+    - Leap seconds happen in winter and summer
+    - DST changes happen in spring and fall
+
+    The largest time zone change that ever happened in a single place
+    was 24 hours:
+    - from SST (UTC-11) - the American Samoa Time Zone
+    - to WST (UTC+13) - the Independent State of Samoa Time Zone
+    The Independent State of Samoa used SST until it moved across
+    the International Date Line at the end of 29 December 2011, to WST.
+    It is now 24 hours ahead of American Samoa
+    (and 25 hours in Southern hemisphere summer).
+    Let's use 24 hours as width (48 hours range total).
+  */
+
+  if (sec < width || sec > TIMESTAMP_MAX_VALUE - width)
+    return false;
+
+  MYSQL_TIME dtmin, dtmax;
+  gmt_sec_to_TIME(&dtmin, sec - width);
+  gmt_sec_to_TIME(&dtmax, sec + width);
+
+  ulonglong seconds;
+  ulong useconds;
+  if (calc_time_diff(&dtmax, &dtmin, 1, &seconds, &useconds))
+    return false; // dtmax is smaller than dtmin, should not happen.
+  if (seconds != (ulonglong) width * 2)
+    return false; // Anomalies found (DST changes or leap seconds)
+  return true;
+}
+
 #endif /* !defined(TESTTIME) && !defined(TZINFO2SQL) */
 
 
@@ -2418,8 +2491,8 @@ print_tz_as_sql(const char* tz_name, const TIME_ZONE_INFO *sp)
     printf("INSERT INTO time_zone_transition \
 (Time_zone_id, Transition_time, Transition_type_id) VALUES\n");
     for (i= 0; i < sp->timecnt; i++)
-      printf("%s(@time_zone_id, %ld, %u)\n", (i == 0 ? " " : ","), sp->ats[i],
-             (uint)sp->types[i]);
+      printf("%s(@time_zone_id, %lld, %u)\n", (i == 0 ? " " : ","),
+             (longlong) sp->ats[i], (uint)sp->types[i]);
     printf(";\n");
   }
 
@@ -2470,8 +2543,8 @@ print_tz_leaps_as_sql(const TIME_ZONE_INFO *sp)
     printf("INSERT INTO time_zone_leap_second \
 (Transition_time, Correction) VALUES\n");
     for (i= 0; i < sp->leapcnt; i++)
-      printf("%s(%ld, %ld)\n", (i == 0 ? " " : ","),
-             sp->lsis[i].ls_trans, sp->lsis[i].ls_corr);
+      printf("%s(%lld, %ld)\n", (i == 0 ? " " : ","),
+             (longlong) sp->lsis[i].ls_trans, sp->lsis[i].ls_corr);
     printf(";\n");
   }
 
