@@ -67,7 +67,7 @@ extern boost::condition_variable cond;
 
 namespace
 {
-[[maybe_unused]] const std::string myname = "DMLProc";
+const std::string myname = "DMLProc";
 }
 
 namespace dmlprocessor
@@ -541,25 +541,6 @@ int PackageHandler::clearTableAccess()
   return 1;
 }
 
-CalpontSystemCatalog::ROPair PackageHandler::getTableRID(
-    boost::shared_ptr<execplan::CalpontSystemCatalog> fcsc,
-    execplan::CalpontSystemCatalog::TableName& tableName)
-{
-  execplan::CalpontSystemCatalog::ROPair roPair;
-  try
-  {
-    roPair = fcsc->tableRID(tableName);
-  }
-  catch (...)
-  {
-    if (setupDec())
-      throw;
-    roPair = fcsc->tableRID(tableName);
-  }
-
-  return roPair;
-}
-
 void PackageHandler::run()
 {
   ResourceManager* frm = ResourceManager::instance();
@@ -596,7 +577,7 @@ void PackageHandler::run()
             CalpontSystemCatalog::TableName tableName;
             tableName.schema = insertPkg.get_Table()->get_SchemaName();
             tableName.table = insertPkg.get_Table()->get_TableName();
-            CalpontSystemCatalog::ROPair roPair = getTableRID(fcsc, tableName);
+            CalpontSystemCatalog::ROPair roPair = fcsc->tableRID(tableName);
             fTableOid = roPair.objnum;
           }
           synchTable.setPackage(this, &insertPkg);  // Blocks if another DML thread is using this fTableOid
@@ -995,7 +976,7 @@ void PackageHandler::run()
             CalpontSystemCatalog::TableName tableName;
             tableName.schema = updatePkg->get_Table()->get_SchemaName();
             tableName.table = updatePkg->get_Table()->get_TableName();
-            CalpontSystemCatalog::ROPair roPair = getTableRID(fcsc, tableName);
+            CalpontSystemCatalog::ROPair roPair = fcsc->tableRID(tableName);
             fTableOid = roPair.objnum;
           }
           synchTable.setPackage(this,
@@ -1055,7 +1036,7 @@ void PackageHandler::run()
             CalpontSystemCatalog::TableName tableName;
             tableName.schema = deletePkg->get_Table()->get_SchemaName();
             tableName.table = deletePkg->get_Table()->get_TableName();
-            CalpontSystemCatalog::ROPair roPair = getTableRID(fcsc, tableName);
+            CalpontSystemCatalog::ROPair roPair = fcsc->tableRID(tableName);
             fTableOid = roPair.objnum;
           }
           synchTable.setPackage(this,
