@@ -65,8 +65,6 @@ public:
 
         const char* table_type() const override;
 
-	const char* index_type(uint key_number) override;
-
 	Table_flags table_flags() const override;
 
 	ulong index_flags(uint idx, uint part, bool all_parts) const override;
@@ -210,16 +208,16 @@ public:
 
 	char* get_foreign_key_create_info() override;
 
-        int get_foreign_key_list(const THD *thd,
+        int get_foreign_key_list(THD *thd,
                                  List<FOREIGN_KEY_INFO> *f_key_list) override;
 
 	int get_parent_foreign_key_list(
-		const THD*		thd,
+		THD*		thd,
 		List<FOREIGN_KEY_INFO>*	f_key_list) override;
 
 	bool can_switch_engines() override;
 
-	uint referenced_by_foreign_key() override;
+	bool referenced_by_foreign_key() const noexcept override;
 
 	void free_foreign_key_create_info(char* str) override { my_free(str); }
 
@@ -934,12 +932,3 @@ ib_push_frm_error(
 @return true if index column length exceeds limit */
 MY_ATTRIBUTE((warn_unused_result))
 bool too_big_key_part_length(size_t max_field_len, const KEY& key);
-
-/** This function is used to rollback one X/Open XA distributed transaction
-which is in the prepared state
-
-@param[in] hton InnoDB handlerton
-@param[in] xid X/Open XA transaction identification
-
-@return 0 or error number */
-int innobase_rollback_by_xid(handlerton* hton, XID* xid);
