@@ -38,17 +38,16 @@ Created 2011/09/02 Sunny Bains
 #include "que0types.h"
 #include "ft_global.h"
 #include "mysql/plugin_ftparser.h"
+#include "lex_string.h"
 
 /** "NULL" value of a document id. */
 #define FTS_NULL_DOC_ID			0
 
 /** FTS hidden column that is used to map to and from the row */
-#define FTS_DOC_ID_COL_NAME		"FTS_DOC_ID"
+static constexpr Lex_cstring FTS_DOC_ID= "FTS_DOC_ID"_LEX_CSTRING;
 
 /** The name of the index created by FTS */
-#define FTS_DOC_ID_INDEX_NAME		"FTS_DOC_ID_INDEX"
-
-#define FTS_DOC_ID_INDEX_NAME_LEN	16
+static constexpr Lex_cstring FTS_DOC_ID_INDEX= "FTS_DOC_ID_INDEX"_LEX_CSTRING;
 
 /** Doc ID is a 8 byte value */
 #define FTS_DOC_ID_LEN			8
@@ -743,21 +742,6 @@ innobase_fts_text_cmp(
 	const void*	p1,			/*!< in: key */
 	const void*	p2);			/*!< in: node */
 
-/******************************************************************//**
-Makes all characters in a string lower case. */
-extern
-size_t
-innobase_fts_casedn_str(
-/*====================*/
-        CHARSET_INFO*	cs,			/*!< in: Character set */
-	char*		src,			/*!< in: string to put in
-						lower case */
-	size_t		src_len,		/*!< in: input string length */
-	char*		dst,			/*!< in: buffer for result
-						string */
-	size_t		dst_len);		/*!< in: buffer size */
-
-
 /******************************************************************//**
 compare two character string according to their charset. */
 extern
