@@ -194,10 +194,14 @@ public:
     WRITE_SYNC= 16,
     /** Asynchronous write */
     WRITE_ASYNC= WRITE_SYNC | 1,
+    /** Asynchronous doublewritten page */
+    WRITE_DBL= WRITE_ASYNC | 4,
     /** A doublewrite batch */
     DBLWR_BATCH= WRITE_ASYNC | 8,
     /** Write data and punch hole for the rest */
     PUNCH= WRITE_ASYNC | 16,
+    /** Write doublewritten data and punch hole for the rest */
+    PUNCH_DBL= PUNCH | 4,
     /** Zero out a range of bytes in fil_space_t::io() */
     PUNCH_RANGE= WRITE_SYNC | 32,
   };
@@ -210,20 +214,27 @@ public:
                       buf_tmp_buffer_t *slot= nullptr) :
     bpage(bpage), slot(slot), type(type) {}
 
-  bool is_read() const noexcept { return (type & READ_SYNC) != 0; }
-  bool is_write() const noexcept { return (type & WRITE_SYNC) != 0; }
-  bool is_async() const noexcept
-  { return (type & (READ_SYNC ^ READ_ASYNC)) != 0; }
+  bool is_read() const { return (type & READ_SYNC) != 0; }
+  bool is_write() const { return (type & WRITE_SYNC) != 0; }
+  bool is_async() const { return (type & (READ_SYNC ^ READ_ASYNC)) != 0; }
+  bool is_doublewritten() const { return (type & 4) != 0; }
 
-  void write_complete(int io_error) const noexcept;
-  void read_complete(int io_error) const noexcept;
-  void fake_read_complete(os_offset_t offset) const noexcept;
+  /** Create a write request for the doublewrite buffer. */
+  IORequest doublewritten() const
+  {
+    ut_ad(type == WRITE_ASYNC || type == PUNCH);
+    return IORequest{bpage, slot, node, Type(type | 4)};
+  }
+
+  void write_complete(int io_error) const;
+  void read_complete(int io_error) const;
+  void fake_read_complete(os_offset_t offset) const;
 
   /** If requested, free storage space associated with a section of the file.
   @param off   byte offset from the start (SEEK_SET)
   @param len   size of the hole in bytes
   @return DB_SUCCESS or error code */
-  dberr_t maybe_punch_hole(os_offset_t off, ulint len) noexcept
+  dberr_t maybe_punch_hole(os_offset_t off, ulint len)
   {
     return off && len && node && (type & (PUNCH ^ WRITE_ASYNC))
       ? punch_hole(off, len)
@@ -235,7 +246,7 @@ private:
   @param off   byte offset from the start (SEEK_SET)
   @param len   size of the hole in bytes
   @return DB_SUCCESS or error code */
-  dberr_t punch_hole(os_offset_t off, ulint len) const noexcept;
+  dberr_t punch_hole(os_offset_t off, ulint len) const;
 
 public:
   /** Page to be written on write operation */
@@ -318,7 +329,8 @@ struct os_file_stat_t {
 the temporary file is created in the in the mysql server configuration
 parameter (--tmpdir).
 @return temporary file handle, or NULL on error */
-FILE *os_file_create_tmpfile() noexcept;
+FILE*
+os_file_create_tmpfile();
 
 /**
 This function attempts to create a directory named pathname. The new directory
@@ -330,8 +342,10 @@ fail_if_exists arguments is true.
 @param[in]	fail_if_exists	if true, pre-existing directory is treated
 				as an error.
 @return true if call succeeds, false on error */
-bool os_file_create_directory(const char *pathname, bool fail_if_exists)
-  noexcept;
+bool
+os_file_create_directory(
+	const char*	pathname,
+	bool		fail_if_exists);
 
 /** NOTE! Use the corresponding macro os_file_create_simple(), not directly
 this function!
@@ -350,7 +364,7 @@ os_file_create_simple_func(
 	os_file_create_t create_mode,
 	ulint		access_type,
 	bool		read_only,
-	bool*		success) noexcept;
+	bool*		success);
 
 /** NOTE! Use the corresponding macro
 os_file_create_simple_no_error_handling(), not directly this function!
@@ -370,7 +384,7 @@ os_file_create_simple_no_error_handling_func(
 	os_file_create_t create_mode,
 	ulint		access_type,
 	bool		read_only,
-	bool*		success) noexcept
+	bool*		success)
 	MY_ATTRIBUTE((warn_unused_result));
 
 #ifndef _WIN32 /* On Microsoft Windows, mandatory locking is used */
@@ -378,7 +392,7 @@ os_file_create_simple_no_error_handling_func(
 @param fd      file descriptor
 @param name    file name
 @return 0 on success */
-int os_file_lock(int fd, const char *name) noexcept;
+int os_file_lock(int fd, const char *name);
 #endif
 
 /** NOTE! Use the corresponding macro os_file_create(), not directly
@@ -398,7 +412,7 @@ os_file_create_func(
 	os_file_create_t create_mode,
 	ulint		type,
 	bool		read_only,
-	bool*		success) noexcept
+	bool*		success)
 	MY_ATTRIBUTE((warn_unused_result));
 
 /** Deletes a file. The file has to be closed before calling this.
@@ -839,7 +853,9 @@ to original un-instrumented file I/O APIs */
 @param[in]	file		handle to a file
 @return file size if OK, else set m_total_size to ~0 and m_alloc_size
 	to errno */
-os_file_size_t os_file_get_size(const char *filename) noexcept
+os_file_size_t
+os_file_get_size(
+	const char*	filename)
 	MY_ATTRIBUTE((warn_unused_result));
 
 /** Determine the logical size of a file.
@@ -854,7 +870,9 @@ os_offset_t os_file_get_size(os_file_t file) noexcept
 /** Truncates a file at its current position.
 @param[in/out]	file	file to be truncated
 @return true if success */
-bool os_file_set_eof(FILE *file) noexcept;
+bool
+os_file_set_eof(
+	FILE*		file);	/*!< in: file to be truncated */
 
 /** Truncate a file to a specified size in bytes.
 @param[in]	pathname	file path
@@ -867,14 +885,16 @@ os_file_truncate(
 	const char*	pathname,
 	os_file_t	file,
 	os_offset_t	size,
-	bool		allow_shrink = false) noexcept;
+	bool		allow_shrink = false);
 
 /** NOTE! Use the corresponding macro os_file_flush(), not directly this
 function!
 Flushes the write buffers of a given file to the disk.
 @param[in]	file		handle to a file
 @return true if success */
-bool os_file_flush_func(os_file_t file) noexcept;
+bool
+os_file_flush_func(
+	os_file_t	file);
 
 /** Retrieves the last error number if an error occurs in a file io function.
 The number should be retrieved before any other OS calls (because they may
@@ -886,7 +906,7 @@ the OS error number + OS_FILE_ERROR_MAX is returned.
                                         to the log
 @return error number, or OS error number + OS_FILE_ERROR_MAX */
 ulint os_file_get_last_error(bool report_all_errors,
-                             bool on_error_silent= false) noexcept;
+                             bool on_error_silent= false);
 
 /** NOTE! Use the corresponding macro os_file_read(), not directly this
 function!
@@ -905,7 +925,7 @@ os_file_read_func(
 	void*			buf,
 	os_offset_t		offset,
 	ulint			n,
-	ulint*			o) noexcept
+	ulint*			o)
 	MY_ATTRIBUTE((warn_unused_result));
 
 /** Rewind file to its start, read at most size - 1 bytes from it to str, and
@@ -918,7 +938,7 @@ void
 os_file_read_string(
 	FILE*		file,
 	char*		str,
-	ulint		size) noexcept;
+	ulint		size);
 
 /** NOTE! Use the corresponding macro os_file_write(), not directly this
 function!
@@ -948,7 +968,7 @@ bool
 os_file_status(
 	const char*	path,
 	bool*		exists,
-	os_file_type_t* type) noexcept;
+	os_file_type_t* type);
 
 /** This function reduces a null-terminated full remote path name into
 the path that is sent by MySQL for DATA DIRECTORY clause.  It replaces
@@ -962,30 +982,34 @@ This function manipulates that path in place.
 If the path format is not as expected, just return.  The result is used
 to inform a SHOW CREATE TABLE command.
 @param[in,out]	data_dir_path		Full path/data_dir_path */
-void os_file_make_data_dir_path(char *data_dir_path) noexcept;
+void
+os_file_make_data_dir_path(
+	char*	data_dir_path);
 
 /** Create all missing subdirectories along the given path.
 @return DB_SUCCESS if OK, otherwise error code. */
-dberr_t os_file_create_subdirs_if_needed(const char* path) noexcept;
+dberr_t
+os_file_create_subdirs_if_needed(
+	const char*	path);
 
 #ifdef UNIV_ENABLE_UNIT_TEST_GET_PARENT_DIR
 /* Test the function os_file_get_parent_dir. */
 void
-unit_test_os_file_get_parent_dir() noexcept;
+unit_test_os_file_get_parent_dir();
 #endif /* UNIV_ENABLE_UNIT_TEST_GET_PARENT_DIR */
 
 /**
 Initializes the asynchronous io system. */
-int os_aio_init() noexcept;
+int os_aio_init();
 
 /**
 Frees the asynchronous io system. */
-void os_aio_free() noexcept;
+void os_aio_free();
 
 /** Submit a fake read request during crash recovery.
 @param type   fake read request
 @param offset additional context */
-void os_fake_read(const IORequest &type, os_offset_t offset) noexcept;
+void os_fake_read(const IORequest &type, os_offset_t offset);
 
 /** Request a read or write.
 @param type		I/O request
@@ -994,34 +1018,36 @@ void os_fake_read(const IORequest &type, os_offset_t offset) noexcept;
 @param n		number of bytes
 @retval DB_SUCCESS if request was queued successfully
 @retval DB_IO_ERROR on I/O error */
-dberr_t os_aio(const IORequest &type, void *buf, os_offset_t offset, size_t n)
-  noexcept;
+dberr_t os_aio(const IORequest &type, void *buf, os_offset_t offset, size_t n);
 
 /** @return number of pending reads */
-size_t os_aio_pending_reads() noexcept;
+size_t os_aio_pending_reads();
 /** @return approximate number of pending reads */
-size_t os_aio_pending_reads_approx() noexcept;
+size_t os_aio_pending_reads_approx();
 /** @return number of pending writes */
-size_t os_aio_pending_writes() noexcept;
+size_t os_aio_pending_writes();
 
 /** Wait until there are no pending asynchronous writes.
 @param declare  whether the wait will be declared in tpool */
-void os_aio_wait_until_no_pending_writes(bool declare) noexcept;
+void os_aio_wait_until_no_pending_writes(bool declare);
 
 /** Wait until all pending asynchronous reads have completed.
 @param declare  whether the wait will be declared in tpool */
-void os_aio_wait_until_no_pending_reads(bool declare) noexcept;
+void os_aio_wait_until_no_pending_reads(bool declare);
 
 /** Prints info of the aio arrays.
 @param[in/out]	file		file where to print */
-void os_aio_print(FILE *file) noexcept;
+void
+os_aio_print(FILE* file);
 
 /** Refreshes the statistics used to print per-second averages. */
-void os_aio_refresh_stats() noexcept;
+void
+os_aio_refresh_stats();
 
 /** Checks that all slots in the system have been freed, that is, there are
 no pending io operations. */
-bool os_aio_all_slots_free() noexcept;
+bool
+os_aio_all_slots_free();
 
 
 /** This function returns information about the specified file
@@ -1036,7 +1062,12 @@ os_file_get_status(
 	const char*	path,
 	os_file_stat_t* stat_info,
 	bool		check_rw_perm,
-	bool		read_only) noexcept;
+	bool		read_only);
+
+/** Set the file create umask
+@param[in]	umask		The umask to use for file creation. */
+void
+os_file_set_umask(ulint umask);
 
 #ifdef _WIN32
 
@@ -1047,7 +1078,7 @@ Make file sparse, on Windows.
 @param[in]	is_sparse if true, make file sparse,
 			otherwise "unsparse" the file
 @return true on success, false on error */
-bool os_file_set_sparse_win32(os_file_t file, bool is_sparse = true) noexcept;
+bool os_file_set_sparse_win32(os_file_t file, bool is_sparse = true);
 
 /**
 Changes file size on Windows
@@ -1064,12 +1095,14 @@ If file is normal, file system allocates storage.
 @param[in]	file		file handle
 @param[in]	size		size to preserve in bytes
 @return true if success */
-bool os_file_set_size(const char *pathname, os_file_t file, os_offset_t size)
-  noexcept;
+bool
+os_file_set_size(
+	const char*	pathname,
+	os_file_t	file,
+	os_offset_t	size);
 
 inline bool
 os_file_set_size(const char* name, os_file_t file, os_offset_t size, bool)
-  noexcept
 {
   return os_file_set_size(name, file, size);
 }
@@ -1093,14 +1126,14 @@ dberr_t
 os_file_punch_hole(
 	os_file_t	fh,
 	os_offset_t	off,
-	os_offset_t	len) noexcept
+	os_offset_t	len)
 	MY_ATTRIBUTE((warn_unused_result));
 
 /* Determine if a path is an absolute path or not.
 @param[in]	OS directory or file path to evaluate
 @retval true if an absolute path
 @retval false if a relative path */
-inline bool is_absolute_path(const char *path) noexcept
+inline bool is_absolute_path(const char *path)
 {
   switch (path[0]) {
 #ifdef _WIN32
@@ -1128,4 +1161,44 @@ inline bool is_absolute_path(const char *path) noexcept
 
 #include "os0file.inl"
 
+/**
+  Structure used for async io statistics
+  There is one instance of this structure for each operation type
+  (read or write)
+*/
+struct innodb_async_io_stats_t
+{
+  /**
+   Current of submitted and not yet finished IOs.
+   IO is considered finished when it finished in the OS
+   *and* the completion callback has been called
+  */
+  size_t pending_ops;
+  /**
+   Time, in seconds, spent waiting for a slot to become
+   available. There is a limited number of slots for async IO
+   operations. If all slots are in use, the IO submission has
+   to wait.
+  */
+  double slot_wait_time_sec;
+
+  /**
+  Information related to IO completion callbacks.
+
+  - number of tasks currently running (<= innodb_read/write_io_threads)
+  - total number of tasks that have been completed
+  - current task queue size . Queueing happens if running tasks is
+    maxed out (equal to innodb_read/write_io_threads)
+  - total number of tasks that have been queued
+  */
+  tpool::group_stats completion_stats;
+};
+
+/**
+  Statistics for asynchronous I/O
+  @param[in] op operation - aio_opcode::AIO_PREAD or aio_opcode::AIO_PWRITE
+  @param[in] stats - structure to fill
+*/
+extern void innodb_io_slots_stats(tpool::aio_opcode op,
+                           innodb_async_io_stats_t *stats);
 #endif /* os0file_h */
