@@ -63,6 +63,8 @@ Created 10/21/1995 Heikki Tuuri
 
 #ifdef _WIN32
 # include <winioctl.h>
+#elif !defined O_DSYNC
+# define O_DSYNC O_SYNC
 #else
 # include <unistd.h>
 #endif
@@ -77,7 +79,7 @@ Created 10/21/1995 Heikki Tuuri
 class io_slots
 {
 private:
-	tpool::cache<tpool::aiocb> m_cache;
+	tpool::cache<tpool::aiocb, true> m_cache;
 	tpool::task_group m_group;
 	int m_max_aio;
 public:
@@ -103,9 +105,9 @@ public:
 	}
 
 	/* Wait for completions of all AIO operations */
-	void wait(mysql_mutex_t &m)
+	void wait(std::unique_lock<std::mutex> &lk)
 	{
-		m_cache.wait(m);
+		m_cache.wait(lk);
 	}
 
 	void wait()
@@ -118,6 +120,11 @@ public:
 		return m_cache.pos();
 	}
 
+	std::chrono::duration<double> wait_time()
+	{
+		return m_cache.wait_time();
+	}
+
 	tpool::task_group* get_task_group()
 	{
 		return &m_group;
@@ -128,7 +135,7 @@ public:
 		wait();
 	}
 
-	mysql_mutex_t& mutex()
+	std::mutex &mutex()
 	{
 		return m_cache.mutex();
 	}
@@ -149,6 +156,20 @@ public:
 static io_slots *read_slots;
 static io_slots *write_slots;
 
+/**
+  Statistics for asynchronous I/O
+  @param op operation type (aio_opcode::AIO_PREAD or aio_opcode::AIO_PWRITE)
+  @param stats pointer to the structure to fill
+*/
+void innodb_io_slots_stats(tpool::aio_opcode op, innodb_async_io_stats_t *stats)
+{
+   io_slots *slots= op == tpool::aio_opcode::AIO_PREAD? read_slots : write_slots;
+
+   stats->pending_ops = slots->pending_io_count();
+   stats->slot_wait_time_sec= slots->wait_time().count();
+   slots->task_group().get_stats(&stats->completion_stats);
+}
+
 /** Number of retries for partial I/O's */
 constexpr ulint NUM_RETRIES_ON_PARTIAL_IO = 10;
 
@@ -940,6 +961,8 @@ bool
 os_file_flush_func(
 	os_file_t	file)
 {
+	if (UNIV_UNLIKELY(my_disable_sync)) return true;
+
 	int	ret;
 
 	ret = os_file_sync_posix(file);
@@ -1002,46 +1025,35 @@ os_file_create_simple_func(
 		}
 	}
 
-	bool	retry;
-
+	if (fil_system.is_write_through()) create_flag |= O_DSYNC;
 #ifdef O_DIRECT
-	int direct_flag = 0;
-	/* This function is always called for data files, we should disable
-	OS caching (O_DIRECT) here as we do in os_file_create_func(), so
-	we open the same file in the same mode, see man page of open(2). */
-	switch (srv_file_flush_method) {
-	case SRV_O_DSYNC:
-	case SRV_O_DIRECT:
-	case SRV_O_DIRECT_NO_FSYNC:
-		direct_flag = O_DIRECT;
-		break;
-	}
+	int direct_flag = fil_system.is_buffered() ? 0 : O_DIRECT;
 #else
 	constexpr int direct_flag = 0;
 #endif
 
-	do {
+	for (;;) {
 		file = open(name, create_flag | direct_flag, os_innodb_umask);
 
 		if (file == -1) {
 #ifdef O_DIRECT
 			if (direct_flag && errno == EINVAL) {
 				direct_flag = 0;
-				retry = true;
 				continue;
 			}
 #endif
-			*success = false;
-			retry = os_file_handle_error_no_exit(
-				name,
-				create_mode == OS_FILE_CREATE
-				? "create" : "open", false);
+
+			if (!os_file_handle_error_no_exit(
+				    name,
+				    create_mode == OS_FILE_CREATE
+				    ? "create" : "open", false)) {
+				break;
+			}
 		} else {
 			*success = true;
-			retry = false;
+			break;
 		}
-
-	} while (retry);
+	}
 
 	if (!read_only
 	    && *success
@@ -1087,7 +1099,7 @@ os_file_create_directory(
 }
 
 #ifdef O_DIRECT
-# if defined __linux
+# ifdef __linux__
 /** Note that the log file uses buffered I/O. */
 static ATTRIBUTE_COLD void os_file_log_buffered()
 {
@@ -1188,14 +1200,8 @@ os_file_create_func(
 	int direct_flag = 0;
 
 	if (type == OS_DATA_FILE) {
-		switch (srv_file_flush_method) {
-		case SRV_O_DSYNC:
-		case SRV_O_DIRECT:
-		case SRV_O_DIRECT_NO_FSYNC:
+		if (!fil_system.is_buffered()) {
 			direct_flag = O_DIRECT;
-			break;
-		default:
-			break;
 		}
 # ifdef __linux__
 	} else if (type == OS_LOG_FILE && create_mode != OS_FILE_CREATE
@@ -1203,11 +1209,6 @@ os_file_create_func(
 		   && !log_sys.is_opened()) {
 		if (stat(name, &st)) {
 			if (errno == ENOENT) {
-				if (create_mode & OS_FILE_ON_ERROR_SILENT) {
-					goto not_found;
-				}
-				sql_print_error(
-					"InnoDB: File %s was not found", name);
 				goto not_found;
 			}
 			log_sys.set_block_size(512);
@@ -1227,17 +1228,11 @@ skip_o_direct:
 	constexpr int direct_flag = 0;
 #endif
 
-
-	/* We let O_DSYNC only affect log files */
-
-	if (!read_only
-	    && type == OS_LOG_FILE
-	    && srv_file_flush_method == SRV_O_DSYNC) {
-#ifdef O_DSYNC
+	if (read_only) {
+	} else if (type == OS_LOG_FILE
+		   ? log_sys.log_write_through
+		   : fil_system.is_write_through()) {
 		create_flag |= O_DSYNC;
-#else
-		create_flag |= O_SYNC;
-#endif
 	}
 
 	os_file_t	file;
@@ -1262,26 +1257,24 @@ skip_o_direct:
 				}
 				continue;
 			}
+# ifdef __linux__
+not_found:
+# endif
 #endif
-			if (!os_file_handle_error_no_exit(
+			if (os_file_handle_error_no_exit(
 				    name, (create_flag & O_CREAT)
 				    ? "create" : "open",
 				    create_mode & OS_FILE_ON_ERROR_SILENT)) {
-				break;
+				continue;
 			}
+
+			return OS_FILE_CLOSED;
 		} else {
 			*success = true;
 			break;
 		}
 	}
 
-	if (!*success) {
-#ifdef __linux__
-not_found:
-#endif
-		return OS_FILE_CLOSED;
-	}
-
 #ifdef __linux__
 	if ((create_flag & O_CREAT) && type == OS_LOG_FILE) {
 		if (fstat(file, &st) || !os_file_log_maybe_unbuffered(st)) {
@@ -1829,6 +1822,9 @@ Flushes the write buffers of a given file to the disk.
 @return true if success */
 bool os_file_flush_func(os_file_t file)
 {
+  if (UNIV_UNLIKELY(my_disable_sync))
+    return true;
+
   ++os_n_fsyncs;
   static bool disable_datasync;
 
@@ -2002,6 +1998,11 @@ os_file_create_simple_func(
 		access = GENERIC_READ | GENERIC_WRITE;
 	}
 
+	if (fil_system.is_write_through())
+		attributes |= FILE_FLAG_WRITE_THROUGH;
+	if (!fil_system.is_buffered())
+		attributes |= FILE_FLAG_NO_BUFFERING;
+
 	for (;;) {
 		/* Use default security attributes and no template file. */
 
@@ -2132,25 +2133,16 @@ os_file_create_func(
 		if (!log_sys.is_opened() && !log_sys.log_buffered) {
 			attributes|= FILE_FLAG_NO_BUFFERING;
 		}
-		if (srv_file_flush_method == SRV_O_DSYNC)
+		if (log_sys.log_write_through)
+			attributes|= FILE_FLAG_WRITE_THROUGH;
+	} else {
+		if (type == OS_DATA_FILE && !fil_system.is_buffered())
+			attributes|= FILE_FLAG_NO_BUFFERING;
+		if (fil_system.is_write_through())
 			attributes|= FILE_FLAG_WRITE_THROUGH;
 	}
-	else if (type == OS_DATA_FILE) {
-		switch (srv_file_flush_method) {
-		case SRV_FSYNC:
-		case SRV_LITTLESYNC:
-		case SRV_NOSYNC:
-			break;
-		default:
-			attributes|= FILE_FLAG_NO_BUFFERING;
-		}
-	}
 
-	DWORD	access = GENERIC_READ;
-
-	if (!read_only) {
-		access |= GENERIC_WRITE;
-	}
+	DWORD access = read_only ? GENERIC_READ : GENERIC_READ | GENERIC_WRITE;
 
 	for (;;) {
 		const  char *operation;
@@ -3361,9 +3353,8 @@ more concurrent threads via thread_group setting.
 int os_aio_resize(ulint n_reader_threads, ulint n_writer_threads)
 {
   /* Lock the slots, and wait until all current IOs finish.*/
-  auto &lk_read= read_slots->mutex(), &lk_write= write_slots->mutex();
-  mysql_mutex_lock(&lk_read);
-  mysql_mutex_lock(&lk_write);
+  std::unique_lock<std::mutex> lk_read(read_slots->mutex()),
+    lk_write(write_slots->mutex());
 
   read_slots->wait(lk_read);
   write_slots->wait(lk_write);
@@ -3391,9 +3382,6 @@ int os_aio_resize(ulint n_reader_threads, ulint n_writer_threads)
     read_slots->resize(max_read_events, static_cast<int>(n_reader_threads));
     write_slots->resize(max_write_events, static_cast<int>(n_writer_threads));
   }
-
-  mysql_mutex_unlock(&lk_read);
-  mysql_mutex_unlock(&lk_write);
   return ret;
 }
 
@@ -3431,10 +3419,8 @@ void os_aio_wait_until_no_pending_writes(bool declare)
 /** @return number of pending reads */
 size_t os_aio_pending_reads()
 {
-  mysql_mutex_lock(&read_slots->mutex());
-  size_t pending= read_slots->pending_io_count();
-  mysql_mutex_unlock(&read_slots->mutex());
-  return pending;
+  std::lock_guard<std::mutex> lock(read_slots->mutex());
+  return read_slots->pending_io_count();
 }
 
 /** @return approximate number of pending reads */
@@ -3446,10 +3432,8 @@ size_t os_aio_pending_reads_approx()
 /** @return number of pending writes */
 size_t os_aio_pending_writes()
 {
-  mysql_mutex_lock(&write_slots->mutex());
-  size_t pending= write_slots->pending_io_count();
-  mysql_mutex_unlock(&write_slots->mutex());
-  return pending;
+  std::lock_guard<std::mutex> lock(write_slots->mutex());
+  return write_slots->pending_io_count();
 }
 
 /** Wait until all pending asynchronous reads have completed.
