@@ -30,17 +30,14 @@ typedef struct st_my_ft_parser_param
   MEM_ROOT *mem_root;
 } MY_FT_PARSER_PARAM;
 
-static int FT_WORD_cmp(void *cs_, const void *w1_, const void *w2_)
+static int FT_WORD_cmp(CHARSET_INFO* cs, FT_WORD *w1, FT_WORD *w2)
 {
-  CHARSET_INFO *cs= cs_;
-  const FT_WORD *w1= w1_, *w2= w2_;
-  return ha_compare_word(cs, w1->pos, w1->len, w2->pos, w2->len);
+  return ha_compare_word(cs, (uchar*) w1->pos, w1->len,
+                             (uchar*) w2->pos, w2->len);
 }
 
-static int walk_and_copy(void *word_, element_count count, void *docstat_)
+static int walk_and_copy(FT_WORD *word,uint32 count,FT_DOCSTAT *docstat)
 {
-    FT_WORD *word= word_;
-    FT_DOCSTAT *docstat= docstat_;
     word->weight=LWS_IN_USE;
     docstat->sum+=word->weight;
     memcpy((docstat->list)++, word, sizeof(FT_WORD));
@@ -61,7 +58,7 @@ FT_WORD * ft_linearize(TREE *wtree, MEM_ROOT *mem_root)
     docstat.list=wlist;
     docstat.uniq=wtree->elements_in_tree;
     docstat.sum=0;
-    tree_walk(wtree,&walk_and_copy,&docstat,left_root_right);
+    tree_walk(wtree,(tree_walk_action)&walk_and_copy,&docstat,left_root_right);
   }
   delete_tree(wtree, 0);
   if (!wlist)
@@ -260,8 +257,8 @@ void ft_parse_init(TREE *wtree, CHARSET_INFO *cs)
 {
   DBUG_ENTER("ft_parse_init");
   if (!is_tree_inited(wtree))
-    init_tree(wtree, 0, 0, sizeof(FT_WORD), &FT_WORD_cmp, 0, (void *) cs,
-              MYF(0));
+    init_tree(wtree, 0, 0, sizeof(FT_WORD), (qsort_cmp2)&FT_WORD_cmp, 0,
+              (void*)cs, MYF(0));
   DBUG_VOID_RETURN;
 }
 
@@ -410,7 +407,7 @@ void ftparser_call_deinitializer(MI_INFO *info)
     {
       MYSQL_FTPARSER_PARAM *ftparser_param=
         &info->ftparser_param[keyinfo->ftkey_nr * MAX_PARAM_NR + j];
-      if (keyinfo->flag & HA_FULLTEXT && ftparser_param->mysql_add_word)
+      if (keyinfo->key_alg == HA_KEY_ALG_FULLTEXT && ftparser_param->mysql_add_word)
       {
         if (keyinfo->parser->deinit)
           keyinfo->parser->deinit(ftparser_param);
