@@ -166,7 +166,7 @@ typedef start_new_trans *SPIDER_Open_tables_backup;
 #define SPIDER_TMP_SHARE_LONG_COUNT         19
 #define SPIDER_TMP_SHARE_LONGLONG_COUNT      3
 
-#define SPIDER_MEM_CALC_LIST_NUM           314
+#define SPIDER_MEM_CALC_LIST_NUM           SPD_MID_LAST
 #define SPIDER_CONN_META_BUF_LEN           64
 
 /*
@@ -181,6 +181,10 @@ typedef start_new_trans *SPIDER_Open_tables_backup;
 */
 enum spider_malloc_id {
   SPD_MID_CHECK_HS_PK_UPDATE_1,
+  SPD_MID_CONN_INIT_1,
+  SPD_MID_CONN_INIT_2,
+  SPD_MID_CONN_QUEUE_AND_MERGE_LOOP_CHECK_1,
+  SPD_MID_CONN_QUEUE_LOOP_CHECK_1,
   SPD_MID_COPY_TABLES_BODY_1,
   SPD_MID_COPY_TABLES_BODY_2,
   SPD_MID_COPY_TABLES_BODY_3,
@@ -287,6 +291,9 @@ enum spider_malloc_id {
   SPD_MID_DB_STORE_RESULT_4,
   SPD_MID_DB_STORE_RESULT_5,
   SPD_MID_DB_STORE_RESULT_FOR_REUSE_CURSOR_1,
+  SPD_MID_DB_STORE_RESULT_FOR_REUSE_CURSOR_2,
+  SPD_MID_DB_STORE_RESULT_FOR_REUSE_CURSOR_3,
+  SPD_MID_DB_STORE_RESULT_FOR_REUSE_CURSOR_4,
   SPD_MID_DB_UDF_COPY_TABLES_1,
   SPD_MID_DB_UDF_PING_TABLE_1,
   SPD_MID_DB_UDF_PING_TABLE_2,
@@ -438,7 +445,8 @@ enum spider_malloc_id {
   SPD_MID_UDF_DIRECT_SQL_CREATE_CONN_KEY_1,
   SPD_MID_UDF_DIRECT_SQL_CREATE_TABLE_LIST_1,
   SPD_MID_UDF_DIRECT_SQL_CREATE_TABLE_LIST_2,
-  SPD_MID_UDF_GET_COPY_TGT_TABLES_1
+  SPD_MID_UDF_GET_COPY_TGT_TABLES_1,
+  SPD_MID_LAST
 };
 
 #define SPIDER_BACKUP_DASTATUS \
@@ -662,6 +670,7 @@ typedef struct st_spider_conn
   char               *tgt_dsn;
   char               *tgt_filedsn;
   char               *tgt_driver;
+  char               *tgt_odbc_conn_str;
   long               tgt_port;
   long               tgt_ssl_vsc;
 
@@ -681,6 +690,7 @@ typedef struct st_spider_conn
   uint               tgt_dsn_length;
   uint               tgt_filedsn_length;
   uint               tgt_driver_length;
+  uint               tgt_odbc_conn_str_length;
   uint               dbton_id;
 
   volatile
@@ -1179,6 +1189,7 @@ typedef struct st_spider_share
   char               **tgt_dsns;
   char               **tgt_filedsns;
   char               **tgt_drivers;
+  char               *tgt_odbc_conn_str;
   char               **static_link_ids;
   char               **tgt_pk_names;
   char               **tgt_sequence_names;
@@ -1221,6 +1232,7 @@ typedef struct st_spider_share
   uint               *tgt_dsns_lengths;
   uint               *tgt_filedsns_lengths;
   uint               *tgt_drivers_lengths;
+  uint               tgt_odbc_conn_str_length;
   uint               *static_link_ids_lengths;
   uint               *tgt_pk_names_lengths;
   uint               *tgt_sequence_names_lengths;
@@ -1445,6 +1457,7 @@ typedef struct st_spider_mon_table_result
 
 typedef struct st_spider_table_mon
 {
+  /* This share has only one link. */
   SPIDER_SHARE               *share;
   uint32                     server_id;
   st_spider_table_mon_list   *parent;
@@ -1471,6 +1484,7 @@ typedef struct st_spider_table_mon_list
   SPIDER_TABLE_MON           *current;
   volatile int               mon_status;
 
+  /* This share has only one link */
   SPIDER_SHARE               *share;
 
   pthread_mutex_t            caller_mutex;
@@ -1484,6 +1498,7 @@ typedef struct st_spider_table_mon_list
 
 typedef struct st_spider_copy_table_conn
 {
+  /* This share has only one link. */
   SPIDER_SHARE               *share;
   int                        link_idx;
   SPIDER_CONN                *conn;
