@@ -514,7 +514,7 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(longlong, split_read)
 static MYSQL_THDVAR_INT(
   semi_split_read, /* name */
   PLUGIN_VAR_RQCMDARG, /* opt */
-  "Use offset and limit parameter in SQL for split_read parameter.", /* comment */
+  "Use offset and limit parameter in SQL for split_read parameter", /* comment */
   NULL, /* check */
   spider_var_deprecated_int, /* update */
   2, /* def */
@@ -660,7 +660,7 @@ static int spider_param_semi_table_lock_check(
     DBUG_RETURN(ER_SPIDER_ALTER_BEFORE_UNLOCK_NUM);
   }
   value->val_int(value, &tmp);
-  options.sub_size = 0;
+  options.deprecation_substitute = 0;
   options.var_type = GET_INT;
   options.def_value = ((MYSQL_SYSVAR_NAME(thdvar_int_t) *) var)->def_val;
   options.min_value = ((MYSQL_SYSVAR_NAME(thdvar_int_t) *) var)->min_val;
@@ -713,7 +713,7 @@ static int spider_param_semi_table_lock_connection_check(
     DBUG_RETURN(ER_SPIDER_ALTER_BEFORE_UNLOCK_NUM);
   }
   value->val_int(value, &tmp);
-  options.sub_size = 0;
+  options.deprecation_substitute = 0;
   options.var_type = GET_INT;
   options.def_value = ((MYSQL_SYSVAR_NAME(thdvar_int_t) *) var)->def_val;
   options.min_value = ((MYSQL_SYSVAR_NAME(thdvar_int_t) *) var)->min_val;
@@ -1317,7 +1317,7 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(double, crd_interval)
 static MYSQL_THDVAR_INT(
   crd_mode, /* name */
   PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
-  "Mode of cardinality confirmation.", /* comment */
+  "Mode of cardinality confirmation", /* comment */
   NULL, /* check */
   spider_var_deprecated_int, /* update */
   1, /* def */
@@ -1338,7 +1338,7 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, crd_mode)
 static MYSQL_THDVAR_INT(
   crd_sync, /* name */
   PLUGIN_VAR_RQCMDARG, /* opt */
-  "Cardinality synchronization in partitioned table.", /* comment */
+  "Cardinality synchronization in partitioned table", /* comment */
   NULL, /* check */
   spider_var_deprecated_int, /* update */
   0, /* def */
@@ -1358,7 +1358,7 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, crd_sync)
 static MYSQL_THDVAR_INT(
   crd_type, /* name */
   PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
-  "Type of cardinality calculation.", /* comment */
+  "Type of cardinality calculation", /* comment */
   NULL, /* check */
   spider_var_deprecated_int, /* update */
   2, /* def */
@@ -1376,7 +1376,7 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, crd_type)
 static MYSQL_THDVAR_INT(
   crd_weight, /* name */
   PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
-  "Weight coefficient to calculate effectiveness of index from cardinality of column.", /* comment */
+  "Weight coefficient to calculate effectiveness of index from cardinality of column", /* comment */
   NULL, /* check */
   spider_var_deprecated_int, /* update */
   2, /* def */
@@ -1396,7 +1396,7 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(double, crd_weight)
 static MYSQL_THDVAR_INT(
   crd_bg_mode, /* name */
   PLUGIN_VAR_RQCMDARG, /* opt */
-  "Mode of cardinality confirmation at background.", /* comment */
+  "Mode of cardinality confirmation at background", /* comment */
   NULL, /* check */
   spider_var_deprecated_int, /* update */
   2, /* def */
@@ -1435,7 +1435,7 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(double, sts_interval)
 static MYSQL_THDVAR_INT(
   sts_mode, /* name */
   PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
-  "Mode of table state confirmation.", /* comment */
+  "Mode of table state confirmation", /* comment */
   NULL, /* check */
   spider_var_deprecated_int, /* update */
   1, /* def */
@@ -1456,7 +1456,7 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, sts_mode)
 static MYSQL_THDVAR_INT(
   sts_sync, /* name */
   PLUGIN_VAR_RQCMDARG, /* opt */
-  "Table state synchronization in partitioned table.", /* comment */
+  "Table state synchronization in partitioned table", /* comment */
   NULL, /* check */
   spider_var_deprecated_int, /* update */
   0, /* def */
@@ -1476,7 +1476,7 @@ SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, sts_sync)
 static MYSQL_THDVAR_INT(
   sts_bg_mode, /* name */
   PLUGIN_VAR_RQCMDARG, /* opt */
-  "Mode of table state confirmation at background.", /* comment */
+  "Mode of table state confirmation at background", /* comment */
   NULL, /* check */
   spider_var_deprecated_int, /* update */
   2, /* def */
@@ -1514,7 +1514,7 @@ SPIDER_THDVAR_VALUE_FUNC(double, ping_interval_at_trx_start)
 static MYSQL_THDVAR_INT(
   auto_increment_mode, /* name */
   PLUGIN_VAR_RQCMDARG, /* opt */
-  "Mode of auto increment.", /* comment */
+  "Mode of auto increment", /* comment */
   NULL, /* check */
   spider_var_deprecated_int, /* update */
   0, /* def */
@@ -1564,7 +1564,7 @@ SPIDER_THDVAR_VALUE_FUNC(bool, local_lock_table)
 static MYSQL_THDVAR_INT(
   use_pushdown_udf, /* name */
   PLUGIN_VAR_RQCMDARG, /* opt */
-  "Remote server transmission existence when UDF is used at condition and \"engine_condition_pushdown=1\"", /* comment */
+  "Remote server transmission existence when UDF is used at condition", /* comment */
   NULL, /* check */
   NULL, /* update */
   0, /* def */
@@ -2041,7 +2041,7 @@ SPIDER_THDVAR_VALUE_FUNC(uint, internal_xa_id_type)
  */
 static MYSQL_THDVAR_INT(
   casual_read, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Read casually if it is possible", /* comment */
   NULL, /* check */
   spider_var_deprecated_int, /* update */
@@ -2214,8 +2214,8 @@ static uint spider_table_sts_thread_count;
 static MYSQL_SYSVAR_UINT(
   table_sts_thread_count,
   spider_table_sts_thread_count,
-  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_READONLY,
-  "Static thread count of table sts",
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_READONLY | PLUGIN_VAR_DEPRECATED,
+  "Deprecated parameter with no effect",
   NULL,
   NULL,
   1,
@@ -2233,8 +2233,8 @@ static uint spider_table_crd_thread_count;
 static MYSQL_SYSVAR_UINT(
   table_crd_thread_count,
   spider_table_crd_thread_count,
-  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_READONLY,
-  "Static thread count of table crd",
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_READONLY | PLUGIN_VAR_DEPRECATED,
+  "Deprecated parameter with no effect",
   NULL,
   NULL,
   1,
@@ -2379,6 +2379,30 @@ static MYSQL_THDVAR_BOOL(
 
 SPIDER_THDVAR_VALUE_FUNC(bool, disable_group_by_handler)
 
+static MYSQL_THDVAR_BOOL(
+  suppress_comment_ignored_warning,
+  PLUGIN_VAR_RQCMDARG,
+  "Whether to suppress warnings that table COMMENT or CONNECTION strings "
+  "are ignored due to specified table options",
+  NULL,
+  NULL,
+  FALSE
+);
+
+SPIDER_THDVAR_VALUE_FUNC(bool, suppress_comment_ignored_warning)
+
+static MYSQL_THDVAR_BOOL(
+  ignore_comments,
+  PLUGIN_VAR_RQCMDARG,
+  "Whether to unconditionally ignore COMMENT and CONNECTION strings "
+  "without checking whether table options are specified",
+  NULL,
+  NULL,
+  FALSE
+);
+
+SPIDER_THDVAR_VALUE_FUNC(bool, ignore_comments)
+
 static struct st_mysql_storage_engine spider_storage_engine =
 { MYSQL_HANDLERTON_INTERFACE_VERSION };
 
@@ -2495,6 +2519,8 @@ static struct st_mysql_sys_var* spider_system_variables[] = {
   MYSQL_SYSVAR(strict_group_by),
   MYSQL_SYSVAR(direct_aggregate),
   MYSQL_SYSVAR(disable_group_by_handler),
+  MYSQL_SYSVAR(suppress_comment_ignored_warning),
+  MYSQL_SYSVAR(ignore_comments),
   NULL
 };
 
