@@ -129,7 +129,7 @@ enum worker_wake_reason
 
 
 /* A per-worker  thread structure.*/
-struct worker_data
+struct alignas(CPU_LEVEL1_DCACHE_LINESIZE)  worker_data
 {
   /** Condition variable to wakeup this worker.*/
   std::condition_variable m_cv;
@@ -156,8 +156,6 @@ struct worker_data
   };
 
   int m_state;
-  /* Padding to avoid false sharing */
-  char m_pad[CPU_LEVEL1_DCACHE_LINESIZE];
 
   bool is_executing_task()
   {
@@ -181,6 +179,16 @@ struct worker_data
     m_state(NONE),
     m_task_start_time()
   {}
+
+  /*Define custom new/delete because of overaligned structure. */
+  static void *operator new(size_t size)
+  {
+    return aligned_malloc(size, CPU_LEVEL1_DCACHE_LINESIZE);
+  }
+  static void operator delete(void* p)
+  {
+    aligned_free(p);
+  }
 };
 
 
@@ -433,7 +441,7 @@ public:
       disarm();
     }
   };
-  timer_generic m_maintenance_timer;
+  timer_generic* m_maintenance_timer=nullptr;
   timer* create_timer(callback_func func, void *data) override
   {
     return new timer_generic(func, data, this);
@@ -564,7 +572,8 @@ void thread_pool_generic::worker_main(worker_data *thread_var)
 {
   task* task;
   set_tls_pool(this);
-  m_worker_init_callback();
+  if(m_worker_init_callback)
+   m_worker_init_callback();
 
   tls_worker_data = thread_var;
   m_thread_creation_pending.clear();
@@ -574,7 +583,8 @@ void thread_pool_generic::worker_main(worker_data *thread_var)
     task->execute();
   }
 
-  m_worker_destroy_callback();
+  if (m_worker_destroy_callback)
+    m_worker_destroy_callback();
 
   worker_end(thread_var);
 }
@@ -741,9 +751,16 @@ bool thread_pool_generic::add_thread()
   reset the flag in thread_pool_generic::worker_main in new thread created. The
   flag must be reset back in case we fail to create the thread. If this flag is
   not reset all future attempt to create thread for this pool would not work as
-  we would return from here. */
-  if (m_thread_creation_pending.test_and_set())
-    return false;
+  we would return from here.
+
+  Do not use this flag for pool of fixed size.
+  (since they lack maintenence that would rectify the pool size, if it is too small)
+  */
+  if (m_min_threads != m_max_threads)
+  {
+    if (m_thread_creation_pending.test_and_set())
+      return false;
+  }
 
   worker_data *thread_data = m_thread_data_cache.get();
   m_active_threads.push_back(thread_data);
@@ -810,13 +827,16 @@ thread_pool_generic::thread_pool_generic(int min_threads, int max_threads) :
   m_min_threads(min_threads),
   m_max_threads(max_threads),
   m_last_thread_count(),
-  m_last_activity(),
-  m_maintenance_timer(thread_pool_generic::maintenance_func, this, nullptr)
+  m_last_activity()
 {
   set_concurrency();
 
   // start the timer
-  m_maintenance_timer.set_time(0, (int)m_timer_interval.count());
+  if (m_min_threads != m_max_threads)
+  {
+    m_maintenance_timer= new timer_generic(thread_pool_generic::maintenance_func, this, nullptr);
+    m_maintenance_timer->set_time(0, (int)m_timer_interval.count());
+  }
 }
 
 
@@ -923,7 +943,8 @@ void thread_pool_generic::switch_timer(timer_state_t state)
   long long period= (state == timer_state_t::OFF) ?
      m_timer_interval.count()*10: m_timer_interval.count();
 
-  m_maintenance_timer.set_period((int)period);
+  if (m_maintenance_timer)
+   m_maintenance_timer->set_period((int)period);
 }
 
 
@@ -941,7 +962,8 @@ thread_pool_generic::~thread_pool_generic()
   m_aio.reset();
 
   /* Also stop the maintanence task early. */
-  m_maintenance_timer.disarm();
+  if (m_maintenance_timer)
+    m_maintenance_timer->disarm();
 
   std::unique_lock<std::mutex> lk(m_mtx);
   m_in_shutdown= true;
@@ -957,6 +979,7 @@ thread_pool_generic::~thread_pool_generic()
   }
 
   lk.unlock();
+  delete m_maintenance_timer;
 }
 
 thread_pool *create_thread_pool_generic(int min_threads, int max_threads)
